//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <lib/ac/stdlib.h>
#include <lib/ac/unistd.h>
#include <libexplain/output.h>
#include <libexplain/program_name.h>
#include <cairomm/surface.h>

#include <lib/calculator/asm.h>
#include <lib/lexer.h>
#include <lib/print_version.h>
#include <lib/program_card/functor/image.h>


static void
usage(void)
{
    const char *prog = explain_program_name_get();
    fprintf(stderr, "Usage: %s [ <option>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


int
main(int argc, char **argv)
{
    for (;;)
    {
        int c = getopt(argc, argv, "I:V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'I':
            lexer::search_path_append(optarg);
            break;

        case 'V':
            print_version();
            return 0;

        default:
            usage();
        }
    }
    if (optind + 2 != argc)
        usage();

    std::string ifn(argv[optind]);
    std::string ofn(argv[optind + 1]);

    calculator::pointer foo = calculator_asm::create();
    foo->program_mode_set(calculator::program_mode_learn);
    foo->load_program(ifn, false);
    program_card_functor_image::pointer pcfp =
        program_card_functor_image::factory(ofn);
    foo->program_card_walk(*pcfp);

    return EXIT_SUCCESS;
}
