//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_CALCULATOR_GNOME_H
#define CANOLA_CALCULATOR_GNOME_H

#include <gtkmm/box.h>
#include <gtkmm/button.h>
#include <gtkmm/label.h>
#include <gtkmm/notebook.h>
#include <gtkmm/table.h>
#include <gtkmm/textview.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/window.h>

#include <lib/calculator.h>

#include <canola/nixie_tubes.h>
#include <canola/retro_button.h>
#include <canola/debugger/gnome.h>
#include <canola/slider/power.h>
#include <canola/slider/program_mode.h>
#include <canola/slider/program_selector.h>
#include <canola/slider/rounding.h>
#include <canola/window/card_reader.h>

class debugger_gnome; // forward
class message_dialog_error; // forward

/**
  * The calculator_gnome class is used to represent the interface to a
  * general state of the 1614p emulation, presented via GNOME widgets.
  */
class calculator_gnome:
    public calculator,
    public Gtk::Window
{
public:
    /**
      * The destructor.
      */
    virtual ~calculator_gnome();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      */
    static pointer create(void);

    // See base class for documentation.
    void run(void);

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

    void program_mode_set_from_dialog(int id);

private:
    // See base class for documentation.
    void derived_print(const display &value);

    // See base class for documentation.
    void derived_feed(void);

    // See base class for documentation.
    void derived_off(void);

    // See base class for documentation.
    void derived_update_display(const display &value);

    // See base class for documentation.
    void clock_tick_start_running(void);

    // See base class for documentation.
    void printer_timer_start_running(void);

    /**
      * The default constructor.
      * It is private on purpose, use the #create class method instead.
      */
    calculator_gnome();

    /**
      * The button_uj is used to remember the state (including
      * appearance) of the widget implementing the UJ button of the
      * emulated calculator.
      */
    retro_button button_uj;

    /**
      * The on_uj method is used to handle the callback when the UJ
      * button is pressed.
      */
    void on_uj(void);

    /**
      * The button_mj is used to remember the state (including
      * appearance) of the widget implementing the MJ button of the
      * emulated calculator.
      */
    retro_button button_mj;

    /**
      * The on_mj method is used to handle the callback when the MJ
      * button is pressed.
      */
    void on_mj(void);

    /**
      * The button_ej is used to remember the state (including
      * appearance) of the widget implementing the EJ button of the
      * emulated calculator.
      */
    retro_button button_ej;

    /**
      * The on_ej method is used to handle the callback when the EJ
      * button is pressed.
      */
    void on_ej(void);

    /**
      * The button_sj is used to remember the state (including
      * appearance) of the widget implementing the SJ button of the
      * emulated calculator.
      */
    retro_button button_sj;

    /**
      * The on_sj method is used to handle the callback when the SJ
      * button is pressed.
      */
    void on_sj(void);

    /**
      * The button_fj is used to remember the state (including
      * appearance) of the widget implementing the FJ button of the
      * emulated calculator.
      */
    retro_button button_fj;

    /**
      * The on_fj method is used to handle the callback when the FJ
      * button is pressed.
      */
    void on_fj(void);

    /**
      * The button_rv is used to remember the state (including
      * appearance) of the widget implementing the RV (Reverse) button
      * of the emulated calculator.
      */
    retro_button button_rv;

    /**
      * The on_rv method is used to handle the callback when the RV
      * (Reverse) button is pressed.
      */
    void on_rv(void);

    /**
      * The button_cm1 is used to remember the state (including
      * appearance) of the widget implementing the CM1 button of the
      * emulated calculator.
      */
    retro_button button_cm1;

    /**
      * The on_cm1 method is used to handle the callback when the CM1
      * button is pressed.
      */
    void on_cm1(void);

    /**
      * The button_c is used to remember the state (including
      * appearance) of the widget implementing the C (Cancel) button of
      * the emulated calculator.
      */
    retro_button button_c;

    /**
      * The on_c method is used to handle the callback when the C
      * (Cancel) button is pressed.
      */
    void on_c(void);

    /**
      * The button_cgh_sign is used to remember the state (including
      * appearance) of the widget implementing the CGH SIGN button of
      * the emulated calculator.
      */
    retro_button button_chg_sign;

    /**
      * The on_chg_sign method is used to handle the callback when the
      * CHG SIGN button is pressed.
      */
    void on_chg_sign(void);

    /**
      * The button_right is used to remember the state (including
      * appearance) of the widget implementing the [->] (Shift Right)
      * button of the emulated calculator.
      */
    retro_button button_right;

    /**
      * The on_right method is used to handle the callback when the [->]
      * (Shift Right) button is pressed.
      */
    void on_right(void);

    /**
      * The button_cm2 is used to remember the state (including
      * appearance) of the widget implementing the CM2 button of the
      * emulated calculator.
      */
    retro_button button_cm2;

    /**
      * The on_cm2 method is used to handle the callback when the CM2
      * button is pressed.
      */
    void on_cm2(void);

    /**
      * The button_clear_indicator instance variable is used to remember
      * the state of the Clear Indicator key's widget.
      */
    retro_button button_clear_indicator;

    /**
      * The on_clear_indicator method is used to handle the callback
      * when the Clear Indicator key is pressed.
      */
    void on_clear_indicator(void);

    /**
      * The button_n7 is used to remember the state (including
      * appearance) of the widget implementing the [7] button of the
      * emulated calculator.
      */
    retro_button button_n7;

    /**
      * The on_n7 method is used to handle the callback when the [7]
      * button is pressed.
      */
    void on_n7(void);

    /**
      * The button_n4 is used to remember the state (including
      * appearance) of the widget implementing the [4] button of the
      * emulated calculator.
      */
    retro_button button_n4;

    /**
      * The on_n4 method is used to handle the callback when the
      * [4] button is pressed.
      */
    void on_n4(void);

    /**
      * The button_n1 is used to remember the state (including
      * appearance) of the widget implementing the [1] button of the
      * emulated calculator.
      */
    retro_button button_n1;

    /**
      * The on_n1 method is used to handle the callback when the
      * [1] button is pressed.
      */
    void on_n1(void);

    /**
      * The button_n0 is used to remember the state (including
      * appearance) of the widget implementing the [0] button of the
      * emulated calculator.
      */
    retro_button button_n0;

    /**
      * The on_n0 method is used to handle the callback when the
      * [0] button is pressed.
      */
    void on_n0(void);

    /**
      * The button_n8 is used to remember the state (including
      * appearance) of the widget implementing the [8] button of the
      * emulated calculator.
      */
    retro_button button_n8;

    /**
      * The on_n8 method is used to handle the callback when the
      * [8] button is pressed.
      */
    void on_n8(void);

    /**
      * The button_n5 is used to remember the state (including
      * appearance) of the widget implementing the [5] button of the
      * emulated calculator.
      */
    retro_button button_n5;

    /**
      * The on_n5 method is used to handle the callback when the
      * [5] button is pressed.
      */
    void on_n5(void);

    /**
      * The button_n2 is used to remember the state (including
      * appearance) of the widget implementing the [2] button of the
      * emulated calculator.
      */
    retro_button button_n2;

    /**
      * The on_n2 method is used to handle the callback when the
      * [2] button is pressed.
      */
    void on_n2(void);

    /**
      * The button_n9 is used to remember the state (including
      * appearance) of the widget implementing the [9] button of the
      * emulated calculator.
      */
    retro_button button_n9;

    /**
      * The on_n9 method is used to handle the callback when the
      * [9] button is pressed.
      */
    void on_n9(void);

    /**
      * The button_n6 is used to remember the state (including
      * appearance) of the widget implementing the [6] button of the
      * emulated calculator.
      */
    retro_button button_n6;

    /**
      * The on_n6 method is used to handle the callback when the
      * [6] button is pressed.
      */
    void on_n6(void);

    /**
      * The button_n3 is used to remember the state (including
      * appearance) of the widget implementing the [3] button of the
      * emulated calculator.
      */
    retro_button button_n3;

    /**
      * The on_n3 method is used to handle the callback when the
      * [3] button is pressed.
      */
    void on_n3(void);

    /**
      * The button_dot is used to remember the state (including
      * appearance) of the widget implementing the [.] button of the
      * emulated calculator.
      */
    retro_button button_dot;

    /**
      * The on_dot method is used to handle the callback when the
      * [.] button is pressed.
      */
    void on_dot(void);

    /**
      * The button_mul is used to remember the state (including
      * appearance) of the widget implementing the [X] button of the
      * emulated calculator.
      */
    retro_button button_mul;

    /**
      * The on_mul method is used to handle the callback when the
      * [X] button is pressed.
      */
    void on_mul(void);

    /**
      * The button_ent is used to remember the state (including
      * appearance) of the widget implementing the [ENT] button of the
      * emulated calculator.
      */
    retro_button button_ent;

    /**
      * The on_ent method is used to handle the callback when the
      * [ENT] button is pressed.
      */
    void on_ent(void);

    /**
      * The button_start is used to remember the state (including
      * appearance) of the widget implementing the [START] button of the
      * emulated calculator.
      */
    retro_button button_start;

    /**
      * The on_start method is used to handle the callback when the
      * [START] button is pressed.
      */
    void on_start(void);

    /**
      * The button_k is used to remember the state (including
      * appearance) of the widget implementing the [K] (Constant) button
      * of the emulated calculator.
      *
      * Instruction Manual, p. 10
      *
      * "This key is for constant multiplication and division.  When it
      * is depressed, it locks and function state is attained.
      *
      * "Internal calculation is performed in the normal procedure.
      * Thereafter constant multiplication and division can be made by
      * operating the numeric and [+=] keys.
      *
      * "In the case of multiplication, the multiplicant is a constant,
      * and in the case of division, the divisor is a constant.
      *
      * "This key is also used in involutions.  The "n"th power of a
      * figure is obtain by depressing the [+=] key (n-1) times.  Thus,
      * a [x] [+=] [+=] [+=] [+=] gives a<sup>5</sup>.
      */
    retro_button button_k;

    /**
      * The on_k_down method is used to handle the callback when the
      * [K] toggle button is pressed.
      */
    void on_k_down(void);

    /**
      * The on_k_up method is used to handle the callback when the
      * [K] button is released.
      */
    void on_k_up(void);

    /**
      * The button_div is used to remember the state (including
      * appearance) of the widget implementing the [/] button of the
      * emulated calculator.
      */
    retro_button button_div;

    /**
      * The on_div method is used to handle the callback when the
      * [/] button is pressed.
      */
    void on_div(void);

    /**
      * The button_sqrt is used to remember the state (including
      * appearance) of the widget implementing the [SQRT] button of the
      * emulated calculator.
      */
    retro_button button_sqrt;

    /**
      * The on_sqrt method is used to handle the callback when the
      * [SQRT] button is pressed.
      */
    void on_sqrt(void);

    /**
      * The button_minus_equals is used to remember the state (including
      * appearance) of the widget implementing the [-=] button of the
      * emulated calculator.
      */
    retro_button button_minus_equals;

    /**
      * The on_minus_equals method is used to handle the callback when
      * the [-=] button is pressed.
      */
    void on_minus_equals(void);

    /**
      * The button_plus_equals is used to remember the state (including
      * appearance) of the widget implementing the [+=] button of the
      * emulated calculator.
      */
    retro_button button_plus_equals;

    /**
      * The on_plus_equals method is used to handle the callback when
      * the [+=] button is pressed.
      */
    void on_plus_equals(void);

    /**
      * Instruction Manual, p. 9
      *
      * "[AM1] Accumulative Memory 1 Key
      *
      * "For automatic accumulation.  Used in sums and differences of products
      * and quotients.  By depressing the [+=] key or [-=] key, the displayed
      * data is added to or subtracted from Memory 1.  This is a lock key for
      * Memory 1.  (Not used in programming.)"
      */
    retro_button button_am1;

    /**
      * The on_am1_down method is used to handle the callback when the
      * [AM1] toggle button is pressed.
      */
    void on_am1_down(void);

    /**
      * The on_am1_up method is used to handle the callback when the
      * [AM1] toggle button is released.
      */
    void on_am1_up(void);

    /**
      * The button_m3 is used to remember the state (including
      * appearance) of the widget implementing the [M3~] button of the
      * emulated calculator.
      */
    retro_button button_m3;

    /**
      * The on_m3_tidle method is used to handle the callback when the
      * [M3~] button is pressed.
      */
    void on_m3_tilde(void);

    /**
      * The button_m2 is used to remember the state (including
      * appearance) of the widget implementing the [M+2] button of the
      * emulated calculator.
      */
    retro_button button_m2;

    /**
      * The on_m2 method is used to handle the callback when the
      * [M+2] button is pressed.
      */
    void on_m2(void);

    /**
      * The button_m1 is used to remember the state (including
      * appearance) of the widget implementing the [M+1] button of the
      * emulated calculator.
      */
    retro_button button_m1;

    /**
      * The on_m1 method is used to handle the callback when the
      * [M+1] button is pressed.
      */
    void on_m1(void);

    /**
      * The button_sm3 is used to remember the state (including
      * appearance) of the widget implementing the [SM3~] button of the
      * emulated calculator.
      */
    retro_button button_sm3;

    /**
      * The on_sm3_tidle method is used to handle the callback when the
      * [SM3~] button is pressed.
      */
    void on_sm3_tilde(void);

    /**
      * The button_mm2 is used to remember the state (including
      * appearance) of the widget implementing the [M-2] button of the
      * emulated calculator.
      */
    retro_button button_mm2;

    /**
      * The on_mm2 method is used to handle the callback when the
      * [M-2] button is pressed.
      */
    void on_mm2(void);

    /**
      * The button_mm1 is used to remember the state (including
      * appearance) of the widget implementing the [M-1] button of the
      * emulated calculator.
      */
    retro_button button_mm1;

    /**
      * The on_mm1 method is used to handle the callback when the
      * [M-1] button is pressed.
      */
    void on_mm1(void);

    /**
      * The button_cm3 is used to remember the state (including
      * appearance) of the widget implementing the [CM3~] button of the
      * emulated calculator.
      */
    retro_button button_cm3;

    /**
      * The on_cm3_tilde method is used to handle the callback when the
      * [CM3~] button is pressed.
      */
    void on_cm3_tilde(void);

    /**
      * The button_rm3 is used to remember the state (including
      * appearance) of the widget implementing the [RM3~] button of the
      * emulated calculator.
      */
    retro_button button_rm3;

    /**
      * The on_rm3_tilde method is used to handle the callback when the
      * [RM3~] button is pressed.
      */
    void on_rm3_tilde(void);

    /**
      * The button_rm2 is used to remember the state (including
      * appearance) of the widget implementing the [RM2] button of the
      * emulated calculator.
      */
    retro_button button_rm2;

    /**
      * The on_rm2 method is used to handle the callback when the
      * [RM2] button is pressed.
      */
    void on_rm2(void);

    /**
      * The button_rm1 is used to remember the state (including
      * appearance) of the widget implementing the [RM1] button of the
      * emulated calculator.
      */
    retro_button button_rm1;

    /**
      * The on_rm1 method is used to handle the callback when the
      * [RM1] button is pressed.
      */
    void on_rm1(void);

    /**
      * The keybopard instance variable is used to rememebr the
      * formatting of all the button widgets in the keyboard.
      */
    Gtk::Table keyboard;

    /**
      * "Power Switch
      *
      * "Power is on when this switch is depressed to ON; and power is off
      * when it is depressed to Off.  When power is turned on, all registers,
      * operation registers and memories are automatically cleared."
      */
    slider_power power_switch;

    Gtk::HBox control_group;

    nixie_tubes display_widget;
    Gtk::Alignment display_widget_aligner;

    Gtk::VBox calculator_layout;

    /**
      * The right_hand_panel instance variable is used to remember the
      * state of the rigth hand panel.  This panel contains the printer,
      * the debugger, and the card system, each as a page in a notebook.
      */
    Gtk::Notebook right_hand_panel;

    /**
      * The printer_label instance variable is used to remember the name
      * of the printer "tab" in the notebook, and its state.
      */
    Gtk::Label printer_label;

    /**
      * The printer_group instance variable is used to remember the
      * widegts that make up the printer, i.e. the paper and the set of
      * buttons.
      */
    Gtk::VBox printer_group;

    /**
      * The printer_paper widget is used to remember the state of the
      * printer paper, including what was printed, and in what color.
      * By using a read-only tesxt view widget, the printer contents can
      * be copied, and pasted into some other application.
      */
    Gtk::TextView printer_paper;

    /**
      * The printer_paper_scroller instance variable is used to rememebr
      * the state of the scroll bar used to control the pinter paper,
      * when the prinout is long enough to need one.
      */
    Gtk::ScrolledWindow printer_paper_scroller;

    /**
      * The printer_button_group instance variable is used to remember
      * the widgets that make up the buttons on the printer.
      */
    Gtk::HBox printer_button_group;

    /**
      * The printer_power instance variable is used to remember the
      * state of the widget representing the printer's power button.
      */
    Gtk::Button printer_power;

    /**
      * The on_printer_power method is used to handle the callback when
      * the printer's Power button is pressed.
      */
    void on_printer_power(void);

    /**
      * The printer_manual_calculation instance variable is used to
      * remember the state of the widget representing the printer's
      * "Manual Calculation" button.
      */
    Gtk::ToggleButton printer_manual_calculation;

    /**
      * The printer_display_print instance variable is used to remember
      * the state of the widget representing the printer's "Display
      * Print" button.
      */
    Gtk::Button printer_display_print;

    /**
      * The on_display_print method is used to handle the callback when the
      * printer's [Display Print] button is pressed.
      */
    void on_display_print(void);

    /**
      * The printer_program_print instance variable is used to remember
      * the state of the widget representing the printer's "Program
      * Print" button.
      */
    Gtk::Button printer_program_print;

    /**
      * The on_program_print method is used to handle the callback when the
      * printer's Program Print button is pressed.
      */
    void on_program_print(void);

    /**
      * The printer_paper_feed instance variable is used to remember
      * the state of the widget representing the printer's "Paper Feed"
      * button.
      */
    Gtk::Button printer_paper_feed;

    /**
      * The on_paper_feed method is used to handle the callback when the
      * printer's Paper Feed button is pressed.
      */
    void on_paper_feed(void);

    /**
      * The debugger_label instance variable is used to remember the
      * name (and state) of the widget in the notebook tab for the
      * debugger.
      */
    Gtk::Label debugger_label;

    /**
      * The debugger_content instance variable is used to remember the
      * state of the Debugger page of the right hand side notebook.
      */
    debugger_gnome debugger_content;

    Gtk::HBox overall_layout;

    /**
      * The on_quit method is used to handle the callback when the
      * [OFF] button is pressed.
      */
    void on_quit(void);

    Glib::RefPtr<Gtk::UIManager> menu_ui;
    Gtk::Widget *menu_bar;
    Gtk::VBox menu_layout;

    /**
      * The on_load_program method is used to implement the File/Program
      * Card System menu item, which is bascically a Load File operation.
      * It invokes a Gnome Open File dialog box,
      * and parses the file, sending the output through the #on_opcode method.
      *
      * Because the #on_opcode method is used, this means that the
      * original calculator is emulated, because it was possible to the
      * the Program Card System to automate key strokes in OPE mode as
      * well.
      */
    void on_load_program(void);

    /**
      * The on_save_program method is used to implement the File/Save
      * Program As menu item.  It invokes a Gnome Save File dialog box,
      * and then asks the base class to do the actual save.
      */
    void on_save_program(void);

    /**
      * The on_save_memories_as_program method is used to implement the
      * "File/Save Memories as Program" menu item.  It invokes a Gnome
      * Save File dialog box, and then asks the base class to do the
      * actual save.
      */
    void on_save_memories_as_program(void);

    /**
      * The do_key_press_event method is used to examine key press
      * events, to see if any of them correspond to a key on the
      * calculator's keyboard, and simulate the button press.
      */
    bool do_key_press_event(GdkEventKey *event);

    /**
      * The do_key_release_event method is used to examine key press
      * events, to see if any of them correspond to a key on the
      * calculator's keyboard, and simulate the button release.
      */
    bool do_key_release_event(GdkEventKey *event);

    /**
      * The program_mode_switch instance variable is used to remember
      * the state of the widget implementing the Program Mode Switch.
      */
    slider_program_mode program_mode_switch;

    /**
      * The program_selector_switch instance variable is used to remember
      * the state of the widget implementing the Program Selector Switch.
      */
    slider_program_selector program_selector_switch;

    /**
      * The rounding_switch instance variable is used to remember
      * the state of the widget implementing the Rounding Switch.
      */
    slider_rounding rounding_switch;

    struct decimal_point_selector_dial_columns_t:
        public Gtk::TreeModel::ColumnRecord
    {
        decimal_point_selector_dial_columns_t()
        {
            add(column_id);
            add(column_name);
        }

        Gtk::TreeModelColumn<int> column_id;
        Gtk::TreeModelColumn<Glib::ustring> column_name;
    };

    /**
      * The decimal_point_selector_dial_columns instance variable is used to
      * remember the tree view model for the Decimal Point Selector Dial widget.
      */
    decimal_point_selector_dial_columns_t decimal_point_selector_dial_columns;

    /**
      * The decimal_point_selector_dial instance variable is used to remember
      * the state of the combo box implementing the Decimal Point Selector Dial.
      */
    Gtk::ComboBox decimal_point_selector_dial;

    /**
      * The decimal_point_selector_dial_tree_model is used to remember the
      * tree model with the actual mode choices in it, that the
      * #decimal_point_selector_dial combo box displays.
      */
    Glib::RefPtr<Gtk::ListStore> decimal_point_selector_dial_tree_model;

private:
    /**
      * The on_program_mode_switch_changed method is used to handle
      * callbacks from the #program_mode_switch widget when the user
      * changes its value.
      */
    void on_program_mode_switch_changed(void);

    /**
      * The on_program_selector_switch_changed method is used to handle
      * callbacks from the #program_selector_switch widget when the user
      * changes its value.
      */
    void on_program_selector_switch_changed(void);

    /**
      * The on_rounding_switch_changed method is used to handle
      * callbacks from the #rounding_switch widget when the user
      * changes its value.
      */
    void on_rounding_switch_changed(void);

    /**
      * The on_decimal_point_selector_dial_changed method is used to
      * handle callbacks from the #decimal_point_selector_dial widget
      * when the user changes its value.
      */
    void on_decimal_point_selector_dial_changed(void);

    /**
      * The on_clock_tick method is used to handle clock tick events
      * delivered by the #clock_tick_timer instance variable.
      */
    bool on_clock_tick(void);

    /**
      * The clock_tick_timer instance variable is used to remember the
      * state of the clock tick timer.
      */
    sigc::connection clock_tick_timer;

    /**
      * The on_printer_finished method is used to handle time printer
      * timer, which emulates the 420ms delay necessary to faithfully
      * emulate the printer behaviour.
      */
    bool on_printer_finished(void);

    /**
      * The printer_timer instance variable is used to remember the
      * state of the printer timer.
      */
    sigc::connection printer_timer;

    /**
      * The error_window instance variable is used to remember the error
      * window, use by on_error to display errors, possibly more than
      * one at a time.
      */
    message_dialog_error *error_window;

    /**
      * The on_opcode method is used to pass an opcode to
      * calculator::on_opcode with a NULL location.
      *
      * @param op
      *     The opcode of interest, usually corresponding to one of the
      *     kayboard keys.
      */
    void on_opcode(opcode_t op);

    /**
      * The on_special_key_symbol method is used to handle a request for
      * the "Edit->Special Key Symbol..." dialog.  It runs the dialog
      * and deals with the result.
      */
    void on_special_key_symbol(void);

    /**
      * The on_edit_preferences method is used to handle a request for
      * the "Edit->Preferences..." dialog.  It runs the dialog and deals
      * with the result.
      */
    void on_edit_preferences(void);

    /**
      * The load_preferences_file method is used to load the user's
      * Canola preferences file, if it exists.
      */
    void load_preferences_file(void);

    /**
      * The save_preferences_file method is used to write the present
      * preferences out to the user's preferences file, creating it if
      * it does not yet exist.
      */
    void save_preferences_file(void);

    /**
      * The user_changed_debugger_address method is a signal handler
      * for when the debugger signals that the user changed the address.
      *
      * @param new_address
      *     The new value of the address field.
      */
    void user_changed_debugger_address(int new_address);

    /**
      * The animate_card_reader instance variable i sused to remember
      * whether or not to animate the card reader.  (If false, the
      * program loads immediately.)
      */
    bool animate_card_reader;

    /**
      * The crp instance variable is used to remember the location of
      * the card reader window.
      */
    window_card_reader::pointer crp;

    /**
      * The copy constructor.
      * Do not use.
      */
    calculator_gnome(const calculator_gnome &);

    /**
      * The assignment operator.
      * Do not use.
      */
    calculator_gnome &operator=(const calculator_gnome &);
};

// vim: set ts=8 sw=4 et :
#endif // CANOLA_CALCULATOR_GNOME_H
