//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_CALCULATOR_PROXY_H
#define CANOLA_CALCULATOR_PROXY_H

#include <lib/calculator.h>

/**
  * The calculator_proxy class is used to represent a half-way house for
  * code, as we animate the crd reader, and feed one opcode at a time,
  * at the same reate as the card reder does on the original.
  */
class calculator_proxy:
    public calculator
{
public:
    /**
      * The destructor.
      */
    virtual ~calculator_proxy();

    /**
      * The constructor.
      */
    calculator_proxy(calculator &deeper);

    /**
      * The forward method is used to forward the instruction at the
      * given address to the deeper instance.  This is how we simulate
      * receiving instructions as the card reading animation progresses.
      */
    void forward(int addr);

protected:
    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

    // See base class for documentation.
    void run(void);

    // See base class for documentation.
    void clock_tick_start_running(void);

    // See base class for documentation.
    void derived_print(const display &value);

    // See base class for documentation.
    void derived_feed(void);

    // See base class for documentation.
    void derived_off(void);

    // See base class for documentation.
    void derived_update_display(const display &value);

    // See base class for documentation.
    void printer_timer_start_running(void);

private:
    /**
      * The deeper instance variable is used to remember the deeper
      * calculator instance that we will forward opcodes to.
      */
    calculator &deeper;

    /**
      * The default constructor.
      * Do not use.
      */
    calculator_proxy();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    calculator_proxy(const calculator_proxy &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    calculator_proxy &operator=(const calculator_proxy &rhs);
};

#endif // CANOLA_CALCULATOR_PROXY_H
