//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_DEBUGGER_GNOME_H
#define CANOLA_DEBUGGER_GNOME_H

#include <lib/config.h>
#include <gtkmm.h>

#include <lib/debugger.h>

/**
  * The debugger_gnome class is used to represent the processing
  * required to display debugging information in a Gtk window.
  */
class debugger_gnome:
    public debugger,
    public Gtk::HBox
{
public:
    /**
      * The destructor.
      */
    virtual ~debugger_gnome();

    /**
      * The default constructor.
      */
    debugger_gnome();

    typedef sigc::signal1<void, int, sigc::nil> address_value_changed_signal_t;

    /**
      * The address_value_changed_signal method is used to obtain a
      * reference to the signal that will be emitted whenever the
      * <i>user</i> changes the address.
      */
    address_value_changed_signal_t &address_value_changed_signal(void);

private:
    // See base class for documentation.
    void display_x_register(const number &value);

    // See base class for documentation.
    void display_y_register(const number &value);

    // See base class for documentation.
    void display_memory(int n, const number &value);

    // See base class for documentation.
    void display_address(int addr);

    // See base class for documentation.
    void display_opcode(opcode_t op);

    // See base class for documentation.
    void display_location(const location::pointer &where);

    // See base class for documentation.
    void display_mstate(const char *text);

private:

    Gtk::Table left_panel;

    Gtk::Label x_reg_name;

    Gtk::TextView x_reg_value;

    Gtk::Label y_reg_name;

    Gtk::TextView y_reg_value;

    Gtk::Label memory_name[14];

    Gtk::TextView memory_value[14];

    Gtk::Label address_name;

    /**
      * The address_value instance variable is used to remember and
      * display the address of the calculator.  It can be changed by the
      * user, to direct execution as desired.
      */
    Gtk::SpinButton address_value;

    /**
      * The address_value_changed_signal_enable instance variable
      * is used to remember whether or not to emit a signal when
      * the #address_value changes.  This has two causes: when the
      * #display_address method is called, and when the user changes it.
      * We want to emit signals for the second, but not the first.
      */
    bool address_value_changed_signal_enable;

    /**
      * The on_address_value_changed method is a signal handler for the
      * value-changed signal of the #address_value widget.  It emits the
      * following signal, when enabled.
      */
    void on_address_value_changed(void);

    /**
      * The address_value_changed_signal_instance instance variable is used to
      * remember the slots connected to this signal.  The emit method will be
      * called by on_address_value_changed when enabled.
      */
    address_value_changed_signal_t address_value_changed_signal_instance;

    Gtk::Label opcode_name;

    Gtk::TextView opcode_value;

    Gtk::Label mstate_name;

    Gtk::TextView mstate_value;

    Gtk::Frame source_code_frame;

    Gtk::ScrolledWindow source_code_scroller;

    Gtk::TextView source_code;

    /**
      * The filename instance variable is used to remember the name of
      * the file currently being displayed in the file text pane.
      */
    std::string filename;

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    debugger_gnome(const debugger_gnome &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    debugger_gnome &operator=(const debugger_gnome &rhs);
};

#endif // CANOLA_DEBUGGER_GNOME_H
