//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_DIALOG_PREFERENCES_H
#define CANOLA_DIALOG_PREFERENCES_H

#include <lib/ac/stddef.h>
#include <gtkmm.h>

class calculator_gnome; // forward

/**
  * The dialog_preferences class is used to represent a windows used
  * select behavioural options.
  */
class dialog_preferences:
    public Gtk::Dialog
{
public:
    /**
      * The destructor.
      */
    virtual ~dialog_preferences();

    /**
      * The default constructor.
      *
      * @param subject
      *     The calculator instance to be manipulated.
      */
    dialog_preferences(calculator_gnome *subject);

    void set_animate_card_reader(bool value);
    bool get_animate_card_reader(void);
    void set_strict_memory_numbers(bool value);
    bool get_strict_memory_numbers(void);

private:
    /**
      * The subject instance variable is used to remember the calculator
      * instance to be manipulated.
      */
    calculator_gnome *subject;

    Gtk::VBox layout;
    Gtk::CheckButton animate_card_reader;
    Gtk::CheckButton strict_memory_numbers;

    /**
      * The default constructor.
      * Do not use.
      */
    dialog_preferences();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    dialog_preferences(const dialog_preferences &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    dialog_preferences &operator=(const dialog_preferences &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // CANOLA_DIALOG_PREFERENCES_H
