//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_DIALOG_PROG_MODE_H
#define CANOLA_DIALOG_PROG_MODE_H

#include <gtkmm/dialog.h>
#include <gtkmm/label.h>

class calculator_gnome; // forward

/**
  * The dialog_prog_mode class is used to represent the UI elements to
  * warn when the program mode is probably wrong, and give the user a
  * chance to change it.
  */
class dialog_prog_mode:
    public Gtk::Dialog
{
public:
    /**
      * The destructor.
      */
    virtual ~dialog_prog_mode();

    /**
      * The constructor.
      *
      * @param subject
      *     The calculator instance to be manipulated.
      */
    dialog_prog_mode(calculator_gnome *subject);

    // See base class for documentation.
    void on_response(int response_id);

private:
    /**
      * The subject instance variable is used to remember the calculator
      * instance to be manipulated.
      */
    calculator_gnome *subject;

    /**
      * The message instance variable is used to rememeber the accumulated text
      * of the error messages.
      */
    Gtk::Label message;

    /**
      * The default constructor.
      * Do not use.
      */
    dialog_prog_mode();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    dialog_prog_mode(const dialog_prog_mode &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    dialog_prog_mode &operator=(const dialog_prog_mode &rhs);
};

// vim: set ts=8 sw=4 et :
#endif // CANOLA_DIALOG_PROG_MODE_H
