//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <canola/calculator/gnome.h>
#include <canola/dialog/special_key_symbol.h>
#include <canola/load_image.h>


dialog_special_key_symbol::~dialog_special_key_symbol()
{
}


dialog_special_key_symbol::dialog_special_key_symbol(
    calculator_gnome *a_subject
) :
    Dialog("Special Key Symbol", *a_subject, true),
    subject(a_subject),
    opcodes(16, 8)
{
    set_icon(load_gui_image("canola"));

    std::string text =
        "The original Canola 1614P allowed you to \"chord\" keys to "
        "enter special key symbols from the keyboard.  This emulator "
        "can't do that, so this dialog is provided instead, so that "
        "you can enter any key symbol value."
        ;
    message.set_text(text);
    message.set_line_wrap(true);
    get_content_area()->add(message);
    message.show();

    Gdk::RGBA red("#FF0000");
    Gdk::RGBA gray("#cc8888");
    for (int y = 0; y < 8; ++y)
    {
        for (int x = 0; x < 16; ++x)
        {
            int n = (y << 4) + x;
            Gtk::Button &b = buttons[n];
            opcode_t op = (opcode_t)n;
            b.set_label(opcode_uname(op));
            b.signal_clicked().connect
            (
                sigc::bind<int>
                (
                    sigc::mem_fun(*this, &dialog_special_key_symbol::response),
                    n
                )
            );

            Gtk::Label *label = dynamic_cast<Gtk::Label *>(b.get_child());
            if (label)
            {
                label->set_justify(Gtk::JUSTIFY_CENTER);
                if (opcode_valid(op))
                {
                    if (opcode_red(op))
                    {
                        label->override_color(red, Gtk::STATE_FLAG_NORMAL);
                        label->override_color(red, Gtk::STATE_FLAG_ACTIVE);
                    }
                }
                else
                {
                    label->override_color(gray, Gtk::STATE_FLAG_NORMAL);
                    label->override_color(gray, Gtk::STATE_FLAG_ACTIVE);
                }
            }
            opcodes.attach(b, x, x + 1, y, y + 1);
        }
    }
    opcodes.show_all();
    get_content_area()->add(opcodes);

    add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
    set_default_response(Gtk::RESPONSE_CANCEL);
}
