//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <lib/ac/stdlib.h>
#include <lib/ac/unistd.h>
#include <gtkmm.h>
#include <libexplain/program_name.h>

#include <lib/lexer.h>
#include <lib/print_version.h>

#include <canola/calculator/gnome.h>
#include <canola/gui_search_path.h>


static void
usage(void)
{
    const char *prog = explain_program_name_get();
    fprintf(stderr, "Usage: %s [ <option>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


int
main(int argc, char **argv)
{
    Gtk::Main kit(argc, argv);
    for (;;)
    {
        int c = getopt(argc, argv, "I:P:V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'I':
            lexer::search_path_append(optarg);
            break;

        case 'P':
            // resource path
            gui_search_path.append(optarg);
            lexer::search_path_append(optarg);
            break;

        case 'V':
            print_version();
            return 0;

        default:
            usage();
        }
    }

    calculator::pointer calc = calculator_gnome::create();

    switch (argc - optind)
    {
    case 0:
        break;

    case 1:
        {
            lexer prog(argv[optind], *calc);
            prog.read_and_execute();
        }
        break;

    default:
        usage();
    }

    calc->run();
    return EXIT_SUCCESS;
}
