//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_MESSAGE_DIALOG_ERROR_H
#define CANOLA_MESSAGE_DIALOG_ERROR_H

#include <gtkmm/messagedialog.h>

/**
  * The message_dialog_error class is used to represent
  */
class message_dialog_error:
    public Gtk::MessageDialog
{
public:
    /**
      * The destructor.
      */
    virtual ~message_dialog_error();

    /**
      * The constructor.
      */
    message_dialog_error(Gtk::Window &w);

    /**
      * The add_text method is used to append yet another error message
      * to the secondary text.
      *
      * @param text
      *     The text of the error message.
      *     It may not include any newline characters.
      */
    void add_text(const std::string &text);

    // See base class for documentation.
    void on_response(int);

private:
    /**
      * The text instance variable is used to remember the full text
      * being displayed in the secondary text area.
      */
    std::string text;

    /**
      * The default constructor.
      * Do not use.
      */
    message_dialog_error();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    message_dialog_error(const message_dialog_error &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    message_dialog_error &operator=(const message_dialog_error &rhs);
};

#endif // CANOLA_MESSAGE_DIALOG_ERROR_H
