//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <canola/nixie_tube.h>
#include <canola/load_image.h>


nixie_tube::~nixie_tube()
{
}


nixie_tube::nixie_tube() :
    pos_number(0)
{
    if (!digit_images[0])
        load_images();

    attach(pos, 0, 1, 0, 1);
    attach(tick, 1, 2, 0, 1);
    attach(digit, 0, 1, 1, 2);
    attach(dot, 1, 2, 1, 2);
    attach(bezlo, 0, 2, 2, 3);

    pos.set(pos_images[0]);
    tick.set(tick_images[0]);
    dot.set(dot_images[0]);
    digit.set(digit_images[10]);
    bezlo.set(bezlo_image);
}


void
nixie_tube::load_images(void)
{
    digit_images[0] = load_gui_image("nixie-tube-32-digit-0");
    digit_images[1] = load_gui_image("nixie-tube-32-digit-1");
    digit_images[2] = load_gui_image("nixie-tube-32-digit-2");
    digit_images[3] = load_gui_image("nixie-tube-32-digit-3");
    digit_images[4] = load_gui_image("nixie-tube-32-digit-4");
    digit_images[5] = load_gui_image("nixie-tube-32-digit-5");
    digit_images[6] = load_gui_image("nixie-tube-32-digit-6");
    digit_images[7] = load_gui_image("nixie-tube-32-digit-7");
    digit_images[8] = load_gui_image("nixie-tube-32-digit-8");
    digit_images[9] = load_gui_image("nixie-tube-32-digit-9");
    digit_images[10] = load_gui_image("nixie-tube-32-digit-off");

    dot_images[0] = load_gui_image("nixie-tube-32-dot-off");
    dot_images[1] = load_gui_image("nixie-tube-32-dot-on");

    tick_images[0] = load_gui_image("nixie-tube-32-tick-blank");
    tick_images[1] = load_gui_image("nixie-tube-32-tick-off");
    tick_images[2] = load_gui_image("nixie-tube-32-tick-on");

    pos_images[0] = load_gui_image("nixie-tube-32-pos-00");
    pos_images[1] = load_gui_image("nixie-tube-32-pos-01");
    pos_images[2] = load_gui_image("nixie-tube-32-pos-02");
    pos_images[3] = load_gui_image("nixie-tube-32-pos-03");
    pos_images[4] = load_gui_image("nixie-tube-32-pos-04");
    pos_images[5] = load_gui_image("nixie-tube-32-pos-05");
    pos_images[6] = load_gui_image("nixie-tube-32-pos-06");
    pos_images[7] = load_gui_image("nixie-tube-32-pos-07");
    pos_images[8] = load_gui_image("nixie-tube-32-pos-08");
    pos_images[9] = load_gui_image("nixie-tube-32-pos-09");
    pos_images[10] = load_gui_image("nixie-tube-32-pos-10");
    pos_images[11] = load_gui_image("nixie-tube-32-pos-11");
    pos_images[12] = load_gui_image("nixie-tube-32-pos-12");
    pos_images[13] = load_gui_image("nixie-tube-32-pos-13");
    pos_images[14] = load_gui_image("nixie-tube-32-pos-14");
    pos_images[15] = load_gui_image("nixie-tube-32-pos-15");
    pos_images[16] = load_gui_image("nixie-tube-32-pos-16");

    bezlo_image = load_gui_image("nixie-tube-32-bezel-lo");
}


void
nixie_tube::set_number(int n)
{
    if (n < 0 || n > 16)
        n = 0;
    pos_number = n;
    pos.set(pos_images[pos_number]);
}


void
nixie_tube::set_value(const display::tube_t &value)
{
    if (value.digit >= 0 && value.digit <= 9)
        digit.set(digit_images[value.digit]);
    else
        digit.set(digit_images[10]);
    dot.set(dot_images[0 + value.dot]);
    if (pos_number <= 1 && !value.tick)
        tick.set(tick_images[0]);
    else
        tick.set(tick_images[1 + value.tick]);
}
