//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_NIXIE_TUBE_H
#define CANOLA_NIXIE_TUBE_H

#include <gtkmm/image.h>
#include <gdkmm/pixbuf.h>
#include <gtkmm/label.h>
#include <gtkmm/table.h>

#include <lib/display.h>


/**
  * The nixie_tube class is used to represent the displayed value of a
  * nixie tube, and its associated indicators.
  */
class nixie_tube:
    public Gtk::Table
{
public:
    /**
      * The destructor.
      */
    virtual ~nixie_tube();

    /**
      * The default constructor.
      */
    nixie_tube();

    void set_value(const display::tube_t &value);
    void set_number(int n);

private:
    typedef Glib::RefPtr<Gdk::Pixbuf> img_ptr_t;

    Gtk::Image pos;
    img_ptr_t pos_images[17];
    int pos_number;

    /**
      * "Punctuation Indicators
      *
      * "They light up automatically at every 3rd digit for punctuation, in
      * accordance with the Decimal Point Selector Dial."
      */
    Gtk::Image tick;

    img_ptr_t tick_images[3];
    Gtk::Image digit;
    img_ptr_t digit_images[11];
    Gtk::Image dot;
    img_ptr_t dot_images[2];
    Gtk::Image bezlo;
    img_ptr_t bezlo_image;

    void load_images(void);

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    nixie_tube(const nixie_tube &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    nixie_tube &operator=(const nixie_tube &rhs);
};

#endif // CANOLA_NIXIE_TUBE_H
