//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <canola/load_image.h>
#include <canola/nixie_tubes.h>


nixie_tubes::~nixie_tubes()
{
}


nixie_tubes::nixie_tubes()
{
    if (!images[0])
        load_images();

    overflow.set(images[0]);
    bezel_left.set(images[3]);
    bezel_right.set(images[4]);
    negative.set(images[5]);

    tube_layout.pack_start(overflow, Gtk::PACK_SHRINK);
    tube_layout.pack_start(bezel_left, Gtk::PACK_SHRINK);
    for (int j = 15; j >= 0; --j)
    {
        tubes[j].set_number(j + 1);
        tube_layout.pack_start(tubes[j], Gtk::PACK_SHRINK);
    }
    tube_layout.pack_start(bezel_right, Gtk::PACK_SHRINK);
    tube_layout.pack_start(negative, Gtk::PACK_SHRINK);

    Gdk::RGBA gray("#404040");
    tube_bg.add(tube_layout);
    tube_bg.override_background_color(gray, Gtk::STATE_FLAG_NORMAL);

    lamp_layout.pack_start(m1_lamp, Gtk::PACK_SHRINK);
    lamp_layout.pack_start(m2_lamp, Gtk::PACK_SHRINK);
    lamp_layout.pack_start(entry_lamp, Gtk::PACK_SHRINK);

    // HBox and VBox don't have an associated drawable, so you can't
    // set their backgrounds.  Instead you have to wrap them in a
    // Gtk::EventBox and set the event box's background.
    lamp_bg.add(lamp_layout);
    lamp_bg.override_background_color(gray, Gtk::STATE_FLAG_NORMAL);

    pack_start(tube_bg, Gtk::PACK_SHRINK);
    pack_start(lamp_bg, Gtk::PACK_SHRINK);
}


void
nixie_tubes::load_images(void)
{
    images[0] = load_gui_image("nixie-tube-32-overflow-blank");
    images[1] = load_gui_image("nixie-tube-32-overflow-off");
    images[2] = load_gui_image("nixie-tube-32-overflow-on");

    images[3] = load_gui_image("nixie-tube-32-bezel-left");
    images[4] = load_gui_image("nixie-tube-32-bezel-right");

    images[5] = load_gui_image("nixie-tube-32-negative-blank");
    images[6] = load_gui_image("nixie-tube-32-negative-off");
    images[7] = load_gui_image("nixie-tube-32-negative-on");

    images[8] = load_gui_image("nixie-tube-32-m1-off");
    images[9] = load_gui_image("nixie-tube-32-m1-on");
    images[10] = load_gui_image("nixie-tube-32-m2-off");
    images[11] = load_gui_image("nixie-tube-32-m2-on");
    images[12] = load_gui_image("nixie-tube-32-entry-off");
    images[13] = load_gui_image("nixie-tube-32-entry-on");
}


void
nixie_tubes::set_value(const display &value)
{
    overflow.set(images[1 + value.overflow]);
    for (unsigned j = 0; j < 16; ++j)
    {
        tubes[j].set_value(value.tubes[j]);
    }
    negative.set(images[6 + value.negative]);

    m1_lamp.set(images[8 + value.m1]);
    m2_lamp.set(images[10 + value.m2]);
    entry_lamp.set(images[12 + value.entry]);
}
