//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <canola/load_image.h>
#include <canola/retro_button.h>


retro_button::~retro_button()
{
}


static img_ptr_t
load_image(const std::string &name, const std::string &state)
{
    std::string filename = "key-" + name + "-" + state;
    return load_gui_image(filename);
}


retro_button::retro_button(
    const char *caption,
    const char *color,
    const Gdk::RGBA &fg,
    bool a_toggle
) :
    text(caption),
    inside(false),
    pressed(false),
    toggle(a_toggle),
    active(false)
{
    normal_image = load_image(color, "normal");
    active_image = load_image(color, "active");
    disabled_image = load_image(color, "disabled");

    text.override_color(fg, Gtk::STATE_FLAG_NORMAL);
    text.override_color(fg, Gtk::STATE_FLAG_ACTIVE);
    text.set_justify(Gtk::JUSTIFY_CENTER);

    appearance.set(normal_image);
    text_aligner.add(text);
    text_aligner.set_size_request(normal_image->get_width() - 1,
        normal_image->get_height() - 1);
    layout.put(appearance, 0, 0);
    layout.put(text_aligner, 0, 0);
    add(layout);

    signal_button_press_event().connect(sigc::mem_fun(*this,
        &retro_button::pressed_callback));
    signal_button_release_event().connect(sigc::mem_fun(*this,
        &retro_button::released_callback));
    signal_enter_notify_event().connect(sigc::mem_fun(*this,
        &retro_button::enter_callback));
    signal_leave_notify_event().connect(sigc::mem_fun(*this,
        &retro_button::leave_callback));

    override_background_color(custom_widget_background, Gtk::STATE_FLAG_NORMAL);
}


bool
retro_button::pressed_callback(const _GdkEventButton *)
{
    pressed = true;
    if (toggle)
    {
        if (active)
            up();
        else
            down();
    }
    else
    {
        down();
    }
    return true;
}


bool
retro_button::released_callback(const _GdkEventButton *)
{
    if (toggle)
    {
        if (pressed && inside)
        {
            if (active)
            {
                signal_released_sender.emit();
                up();
            }
            else
            {
                signal_clicked_sender.emit();
                down();
            }
            active = !active;
        }
        else
        {
            // restore
            if (active)
                down();
            else
                up();
        }
    }
    else
    {
        if (pressed && inside)
        {
            signal_clicked_sender.emit();
        }
        up();
    }
    pressed = false;
    return true;
}


bool
retro_button::enter_callback(const _GdkEventCrossing *)
{
    inside = true;
    if (toggle)
    {
        if (active)
            up();
        else
            down();
    }
    else
    {
        if (pressed)
            down();
        else
            up();
    }
    return true;
}


bool
retro_button::leave_callback(const _GdkEventCrossing *)
{

    inside = false;
    if (toggle)
    {
        if (active)
        {
            down();
        }
        else
        {
            up();
        }
    }
    else
    {
        up();
    }
    return true;
}


void
retro_button::down(void)
{
    appearance.set(active_image);
    layout.move(text_aligner, 1, 1);
}


void
retro_button::up(void)
{
    appearance.set(normal_image);
    layout.move(text_aligner, 0, 0);
}
