//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <libexplain/fclose.h>
#include <libexplain/fflush.h>
#include <libexplain/fopen.h>

#include <lib/calculator.h>


bool
calculator::save_memories_as_program(const std::string &filename)
{
    FILE *fp = fopen(filename.c_str(), "w");
    if (!fp)
    {
        on_error("%s", explain_fopen(filename.c_str(), "w"));
        return false;
    }
    for (unsigned j = 1; j <= 14; ++j)
    {
        const number &n = memory[j];
        if (n != 0)
        {
            std::string s = n.to_string();
            const char *cp = s.c_str();
            const char *ep = cp + s.size();
            while (*cp == ' ')
                ++cp;
            while (*cp == '0')
                ++cp;
            while (ep > cp && ep[-1] == '0')
                --ep;
            if (ep > cp && ep[-1] == '.')
                --ep;
            assert(cp != ep);
            fprintf(fp, "%.*s\n", (int)(ep - cp), cp);
            if (j == 1 || j == 2)
            {
                fprintf(fp, "CM%d\n", j);
                fprintf(fp, "M%d\n", j);
            }
            else
            {
                fprintf(fp, "SM%d\n", j);
            }
        }
    }
    if (fflush(fp))
    {
        on_error("%s", explain_fflush(fp));
        fclose(fp);
        return false;
    }
    if (fclose(fp))
    {
        on_error("%s", explain_fclose(fp));
        return false;
    }
    return true;
}
