//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/calculator.h>


void
calculator::update_display(void)
{
    switch (program_mode)
    {
    case program_mode_learn:
    case program_mode_program_check:
        update_display_lrn();
        break;

    case program_mode_operation:
    case program_mode_operation_check:
        update_display_ope();
        break;

    default:
        assert(!"this mode doesn't exist");
        break;
    }
}


void
calculator::calculate_display_lrn(display &value, int addr, int opc)
{
    value.tubes[9].digit = addr / 100;
    value.tubes[8].digit = (addr / 10) % 10;
    value.tubes[7].digit = addr % 10;

    value.tubes[4].digit = opc >> 4;
    value.tubes[3].digit = (opc & 15) / 10;
    value.tubes[2].digit = (opc & 15) % 10;

    if (!opcode_valid(opc))
        value.negative = true;
    value.set_opcode(opcode_t(opc));

    //
    // At least one tube MUST have a the dot turned on
    // (probably a hardware limitation).
    //
    // <question>
    // What does the dot in digit 16 mean in LRN mode?
    // Does PRO CHE mode have the dot in a different column?
    // </question>
    //
    value.tubes[15].dot = true;
}


void
calculator::update_display_lamps(display &value)
{
    value.m1 = (memory[1] != 0);
    value.m2 = (memory[2] != 0);
    switch (clock_tick_mode)
    {
    case clock_tick_mode_disabled:
    case clock_tick_mode_running:
        break;

    case clock_tick_mode_entry:
    case clock_tick_mode_ej:
    case clock_tick_mode_sj:
       value.entry = true;
       break;

    default:
        break;
    }
}


void
calculator::update_display_lrn(void)
{
    unsigned addr = wrap_address(address);
    assert(addr >= 1 && addr <= 240);

    display value;
    calculate_display_lrn(value, addr, programme[addr]);
    update_display_lamps(value);
    derived_update_display(value);
}


void
calculator::update_display_ope(void)
{
    display value = x_register.get_display();
    update_display_lamps(value);
    derived_update_display(value);
}
