//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/display.h>
#include <lib/sizeof.h>


display::~display()
{
}


display::display() :
    overflow(false),
    negative(false),
    m1(false),
    m2(false),
    entry(false),
    column18(column18_blank),
    column19(column19_blank),
    column20(column20_blank),
    column21(column21_blank)
{
}


display::display(const display &rhs) :
    overflow(rhs.overflow),
    negative(rhs.negative),
    m1(rhs.m1),
    m2(rhs.m2),
    entry(rhs.entry),
    column18(rhs.column18),
    column19(rhs.column19),
    column20(rhs.column20),
    column21(rhs.column21)
{
    for (size_t j = 0; j < SIZEOF(tubes); ++j)
        tubes[j] = rhs.tubes[j];
}


display &
display::operator=(const display &rhs)
{
    if (this != &rhs)
    {
        overflow = rhs.overflow;
        for (size_t j = 0; j < SIZEOF(tubes); ++j)
            tubes[j] = rhs.tubes[j];
        negative = rhs.negative;
        m1 = rhs.m1;
        m2 = rhs.m2;
        entry = rhs.entry;
        column18 = rhs.column18;
        column19 = rhs.column19;
        column20 = rhs.column20;
        column21 = rhs.column21;
    }
    return *this;
}


display::tube_t::~tube_t()
{
}


display::tube_t::tube_t() :
    digit(10),
    dot(false),
    tick(false)
{
}


display::tube_t::tube_t(const tube_t &rhs) :
    digit(rhs.digit),
    dot(rhs.dot),
    tick(rhs.tick)
{
}


display::tube_t &
display::tube_t::operator=(const tube_t &rhs)
{
    if (this != &rhs)
    {
        digit = rhs.digit;
        dot = rhs.dot;
        tick = rhs.tick;
    }
    return *this;
}


void
display::set_opcode(opcode_t op)
{
    column18 = column18_blank;
    column19 = column19_blank;
    column20 = column20_blank;
    column21 = column21_blank;
    switch (op)
    {
    case opcode_zero:
        goto yuck;

    case opcode_plus_equals:
        column19 = column19_plus_equals;
        break;

    case opcode_minus_equals:
        column19 = column19_minus_equals;
        break;

    case opcode_mul:
        column19 = column19_times;
        break;

    case opcode_div:
        column19 = column19_divide;
        break;

    case opcode_sqrt:
        column19 = column19_sqrt;
        break;

    case opcode_ent:
        column18 = column18_e;
        break;

    case opcode_sj:
        column18 = column18_sj;
        break;

    case opcode_ej:
        column18 = column18_ej;
        break;

    case opcode_mj:
        column18 = column18_mj;
        break;

    case opcode_uj:
        column18 = column18_uj;
        break;

    case opcode_fj:
        column18 = column18_fj;
        break;

    case opcode_suj:
        column18 = column18_suj;
        break;

    case opcode_sfj:
        column18 = column18_sfj;
        break;

    case opcode_srj:
        column18 = column18_srj;
        break;

    case opcode_m1:
    case opcode_m2:
    case opcode_m3:
    case opcode_m4:
    case opcode_m5:
    case opcode_m6:
    case opcode_m7:
    case opcode_m8:
    case opcode_m9:
    case opcode_m10:
    case opcode_m11:
    case opcode_m12:
    case opcode_m13:
    case opcode_m14:
        {
            int n = op - (opcode_m1 - 1);
            column19 = column19_m_plus;
            if (n >= 10)
            {
                column20 = column20_n1;
                n -= 10;
            }
            column21 = column21_t(column21_n0 + n);
        }
        break;

    case opcode_mm1:
    case opcode_mm2:
        column19 = column19_m_minus;
        column21 = column21_t(column21_n0 + (op - (opcode_mm1 - 1)));
        break;

    case opcode_sm3:
    case opcode_sm4:
    case opcode_sm5:
    case opcode_sm6:
    case opcode_sm7:
    case opcode_sm8:
    case opcode_sm9:
    case opcode_sm10:
    case opcode_sm11:
    case opcode_sm12:
    case opcode_sm13:
    case opcode_sm14:
        {
            int n = op - (opcode_sm3 - 3);
            column19 = column19_sm;
            if (n >= 10)
            {
                column20 = column20_n1;
                n -= 10;
            }
            column21 = column21_t(column21_n0 + n);
        }
        break;

    case opcode_rm1:
    case opcode_rm2:
    case opcode_rm3:
    case opcode_rm4:
    case opcode_rm5:
    case opcode_rm6:
    case opcode_rm7:
    case opcode_rm8:
    case opcode_rm9:
    case opcode_rm10:
    case opcode_rm11:
    case opcode_rm12:
    case opcode_rm13:
    case opcode_rm14:
        {
            int n = op - (opcode_rm1 - 1);
            column19 = column19_rm;
            if (n >= 10)
            {
                column20 = column20_n1;
                n -= 10;
            }
            column21 = column21_t(column21_n0 + n);
        }
        break;

    case opcode_clear_indicator:
        column18 = column18_ce;
        break;

    case opcode_cm1:
    case opcode_cm2:
    case opcode_cm3:
    case opcode_cm4:
    case opcode_cm5:
    case opcode_cm6:
    case opcode_cm7:
    case opcode_cm8:
    case opcode_cm9:
    case opcode_cm10:
    case opcode_cm11:
    case opcode_cm12:
    case opcode_cm13:
    case opcode_cm14:
        {
            int n = op - (opcode_cm1 - 1);
            column18 = column18_cm;
            if (n >= 10)
            {
                column20 = column20_n1;
                n -= 10;
            }
            column21 = column21_t(column21_n0 + n);
        }
        break;

    case opcode_dot:
        column20 = column20_dot;
        break;

    case opcode_rv:
        column18 = column18_rv;
        break;

    case opcode_right:
        column18 = column18_right;
        break;

    case opcode_chg_sign:
        column18 = column18_cs;
        break;

    case opcode_round_down:
        column20 = column20_round_down;
        break;

    case opcode_round_off:
        column20 = column20_round_off;
        break;

    case opcode_round_up:
        column20 = column20_round_up;
        break;

    case opcode_print:
        column19 = column19_print;
        break;

    case opcode_fd:
        column20 = column20_fd;
        break;

    case opcode_n0:
    case opcode_n1:
    case opcode_n2:
    case opcode_n3:
    case opcode_n4:
    case opcode_n5:
    case opcode_n6:
    case opcode_n7:
    case opcode_n8:
    case opcode_n9:
    case opcode_n10:
    case opcode_n11:
    case opcode_n12:
    case opcode_n13:
    case opcode_n14:
        {
            int n = op - opcode_n0;
            if (n >= 10)
            {
                column20 = column20_n1;
                n -= 10;
            }
            column21 = column21_t(column21_n0 + n);
        }
        break;

    case opcode_extended_c:
    case opcode_extended_start:
    case opcode_extended_m3_tilde:
    case opcode_extended_sm3_tilde:
    case opcode_extended_rm3_tilde:
    case opcode_extended_cm3_tilde:
    case opcode_extended_off:
    case opcode_extended_program_print:
    case opcode_extended_display_print:
    case opcode_extended_paper_feed:
    case opcode_extended_rounding_switch_up:
    case opcode_extended_rounding_switch_off:
    case opcode_extended_rounding_switch_down:
    case opcode_extended_decimal_point_selector_0:
    case opcode_extended_decimal_point_selector_1:
    case opcode_extended_decimal_point_selector_2:
    case opcode_extended_decimal_point_selector_3:
    case opcode_extended_decimal_point_selector_4:
    case opcode_extended_decimal_point_selector_5:
    case opcode_extended_decimal_point_selector_6:
    case opcode_extended_decimal_point_selector_7:
    case opcode_extended_decimal_point_selector_8:
    case opcode_extended_decimal_point_selector_9:
    case opcode_extended_decimal_point_selector_10:
    case opcode_extended_decimal_point_selector_11:
    case opcode_extended_decimal_point_selector_12:
    case opcode_extended_decimal_point_selector_float:
    case opcode_extended_goto_000:
    case opcode_extended_goto_001:
    case opcode_extended_goto_002:
    case opcode_extended_goto_003:
    case opcode_extended_goto_004:
    case opcode_extended_goto_005:
    case opcode_extended_goto_006:
    case opcode_extended_goto_007:
    case opcode_extended_goto_008:
    case opcode_extended_goto_009:
    case opcode_extended_goto_010:
    case opcode_extended_goto_011:
    case opcode_extended_goto_012:
    case opcode_extended_goto_013:
    case opcode_extended_goto_014:
    case opcode_extended_goto_015:
    case opcode_extended_goto_100:
    case opcode_extended_goto_101:
    case opcode_extended_goto_102:
    case opcode_extended_goto_103:
    case opcode_extended_goto_104:
    case opcode_extended_goto_105:
    case opcode_extended_goto_106:
    case opcode_extended_goto_107:
    case opcode_extended_goto_108:
    case opcode_extended_goto_109:
    case opcode_extended_goto_110:
    case opcode_extended_goto_111:
    case opcode_extended_goto_112:
    case opcode_extended_goto_113:
    case opcode_extended_goto_114:
    case opcode_extended_goto_115:
    case opcode_extended_goto_200:
    case opcode_extended_goto_201:
    case opcode_extended_goto_202:
    case opcode_extended_goto_203:
    case opcode_extended_goto_204:
    case opcode_extended_goto_205:
    case opcode_extended_goto_206:
    case opcode_extended_goto_207:
    case opcode_extended_goto_208:
    case opcode_extended_goto_209:
    case opcode_extended_goto_210:
    case opcode_extended_goto_211:
    case opcode_extended_goto_212:
    case opcode_extended_goto_213:
    case opcode_extended_goto_214:
    case opcode_extended_goto_215:
    case opcode_extended_goto_300:
    case opcode_extended_goto_301:
    case opcode_extended_goto_302:
    case opcode_extended_goto_303:
    case opcode_extended_goto_304:
    case opcode_extended_goto_305:
    case opcode_extended_goto_306:
    case opcode_extended_goto_307:
    case opcode_extended_goto_308:
    case opcode_extended_goto_309:
    case opcode_extended_goto_310:
    case opcode_extended_goto_311:
    case opcode_extended_goto_312:
    case opcode_extended_goto_313:
    case opcode_extended_goto_314:
    case opcode_extended_goto_315:
    case opcode_extended_goto_400:
    case opcode_extended_goto_401:
    case opcode_extended_goto_402:
    case opcode_extended_goto_403:
    case opcode_extended_goto_404:
    case opcode_extended_goto_405:
    case opcode_extended_goto_406:
    case opcode_extended_goto_407:
    case opcode_extended_goto_408:
    case opcode_extended_goto_409:
    case opcode_extended_goto_410:
    case opcode_extended_goto_411:
    case opcode_extended_goto_412:
    case opcode_extended_goto_413:
    case opcode_extended_goto_414:
    case opcode_extended_goto_415:
    case opcode_extended_goto_500:
    case opcode_extended_goto_501:
    case opcode_extended_goto_502:
    case opcode_extended_goto_503:
    case opcode_extended_goto_504:
    case opcode_extended_goto_505:
    case opcode_extended_goto_506:
    case opcode_extended_goto_507:
    case opcode_extended_goto_508:
    case opcode_extended_goto_509:
    case opcode_extended_goto_510:
    case opcode_extended_goto_511:
    case opcode_extended_goto_512:
    case opcode_extended_goto_513:
    case opcode_extended_goto_514:
    case opcode_extended_goto_515:
    case opcode_extended_goto_600:
    case opcode_extended_goto_601:
    case opcode_extended_goto_602:
    case opcode_extended_goto_603:
    case opcode_extended_goto_604:
    case opcode_extended_goto_605:
    case opcode_extended_goto_606:
    case opcode_extended_goto_607:
    case opcode_extended_goto_608:
    case opcode_extended_goto_609:
    case opcode_extended_goto_610:
    case opcode_extended_goto_611:
    case opcode_extended_goto_612:
    case opcode_extended_goto_613:
    case opcode_extended_goto_614:
    case opcode_extended_goto_615:
    case opcode_extended_goto_700:
    case opcode_extended_goto_701:
    case opcode_extended_goto_702:
    case opcode_extended_goto_703:
    case opcode_extended_goto_704:
    case opcode_extended_goto_705:
    case opcode_extended_goto_706:
    case opcode_extended_goto_707:
    case opcode_extended_goto_708:
    case opcode_extended_goto_709:
    case opcode_extended_goto_710:
    case opcode_extended_goto_711:
    case opcode_extended_goto_712:
    case opcode_extended_goto_713:
    case opcode_extended_goto_714:
    case opcode_extended_goto_715:
        // nothing for extended opcodes
        break;

    default:
        yuck:
        if ((int)op < 128)
        {
            // used to indicate an invalid opcode
            column18 = column18_minus;
        }
        break;
    }
}
