//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_DISPLAY_H
#define LIB_DISPLAY_H

#include <lib/opcode.h>

/**
  * The display class is used to represent the state of the display.
  * This is used both for updating the display and for printing.
  */
class display
{
public:
    struct tube_t
    {
        ~tube_t();
        tube_t();
        tube_t(const tube_t &rhs);
        tube_t &operator=(const tube_t &rhs);
        int digit: 6;
        bool dot: 1;
        bool tick: 1;
    };

    ~display();
    display();
    display(const display &rhs);
    display &operator=(const display &rhs);

    bool overflow;
    tube_t tubes[16];
    bool negative;
    bool m1;
    bool m2;
    bool entry;

    /**
      * The set_opcode method is used to set the right hand columns to
      * indicate the opcode being performed, or for other informative
      * reasons.
      *
      * @param op
      *     The opcode of interest.
      */
    void set_opcode(opcode_t op);

    enum column18_t
    {
        column18_blank,
        column18_minus,
        column18_ce,
        column18_cm,
        column18_e,
        column18_fj,
        column18_ej,
        column18_uj,
        column18_mj,
        column18_sj,
        column18_rv, // uncertain
        column18_sfj, // uncertain
        column18_suj, // uncertain
        column18_srj, // uncertain
        column18_right, // uncertain
        column18_cs, // uncertain
    };

    column18_t column18;

    enum column19_t
    {
        column19_blank,
        column19_plus_equals,
        column19_minus_equals,
        column19_times,
        column19_divide,
        column19_sqrt,
        column19_m_plus,
        column19_m_minus,
        column19_rm,
        column19_sm,
        column19_print,
    };

    column19_t column19;

    enum column20_t
    {
        column20_blank,
        column20_n1,
        column20_fd,
        column20_round_up,
        column20_round_off,
        column20_round_down,
        column20_dot,
    };

    column20_t column20;

    enum column21_t
    {
        column21_blank,
        column21_n0,
        column21_n1,
        column21_n2,
        column21_n3,
        column21_n4,
        column21_n5,
        column21_n6,
        column21_n7,
        column21_n8,
        column21_n9,
    };

    column21_t column21;
};

#endif // LIB_DISPLAY_H
