//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/string.h>

#include <lib/image/format.h>


image_format::~image_format()
{
    delete [] data;
}


image_format::image_format(int a_width, int a_height, data_t *a_data) :
    width(a_width),
    height(a_height),
    data(a_data)
{
    assert(width >= 1);
    assert(height >= 1);
    assert(!!data);
}


image_format::image_format(int a_width, int a_height) :
    width(a_width),
    height(a_height),
    data(0)
{
    size_t nbytes = 4 * width * height;
    data = new unsigned char [nbytes];
    memset(data, 0, nbytes);
}


image_format &
image_format::operator=(const image_format &arg)
{
    if (this != &arg)
    {
        delete [] data;
        width = arg.width;
        height = arg.height;
        size_t nbytes = 4 * width * height;
        data = new unsigned char [nbytes];
        memcpy(data, arg.data, nbytes);
    }
    return *this;
}


unsigned
image_format::get_width(void)
    const
{
    return width;
}


unsigned
image_format::get_height(void)
    const
{
    return height;
}
