//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_FORMAT_JPEG_H
#define LIB_IMAGE_FORMAT_JPEG_H

#include <lib/image/format.h>


/**
  * The image_format_jpeg class is used to represent an image in JPEG
  * format.
  */
class image_format_jpeg:
    public image_format
{
public:
    /**
      * The destructor.
      */
    virtual ~image_format_jpeg();

    /**
      * The create class method is used to create an image by reading a
      * file.
      *
      * @param filename
      *     The name of the file to be read.
      */
    static pointer create(const std::string &filename);

    /**
      * The create_blank class method is used to create a new image
      * which is entirely blank (and transparent).
      *
      * @param width
      *     The width of the image
      * @param height
      *     The height of the image
      */
    static pointer create_blank(int width, int height);

protected:
    // See base class for documentation.
    const char *type_name() const;

    // See base class for documentation.
    void save_to_file(const std::string &filename) const;

private:
    /**
      * The default constructor.  Do not use.
      */
    image_format_jpeg();

    /**
      * The constructor.  It is private on purpose, use the "create"
      * class method instead.
      *
      * @param width
      *     The width of the new image
      * @param height
      *     The height of the new image
      * @param data
      *     The data of the new image, indexed row then column then
      *     component (RGBA).
      */
    image_format_jpeg(int width, int height, data_t *data);

    /**
      * The constructor.  It is private on purpose, use the
      * "create_blank" class method instead.
      *
      * @param width
      *     The width of the new image
      * @param height
      *     The height of the new image
      */
    image_format_jpeg(int width, int height);

    /**
      * The copy constructor.  Do not use.
      */
    image_format_jpeg(const image_format_jpeg &);

    /**
      * The assignment operator.  Do not use.
      */
    image_format_jpeg &operator=(const image_format_jpeg &);
};

#endif // LIB_IMAGE_FORMAT_JPEG_H
