//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/image/invert.h>


image_invert::~image_invert()
{
}


image_invert::image_invert(const image::pointer &a_deeper) :
    deeper(a_deeper)
{
}


image_invert::pointer
image_invert::create(const image::pointer &a_deeper)
{
    return pointer(new image_invert(a_deeper));
}


const char *
image_invert::type_name(void)
    const
{
    return deeper->type_name();
}


void
image_invert::get_pixel(int x, int y, icon_pixel &result)
    const
{
    icon_pixel tmp;
    deeper->get_pixel(x, y, tmp);
    result =
        icon_pixel
        (
            1. - tmp.get_red(),
            1. - tmp.get_green(),
            1. - tmp.get_blue(),
            tmp.get_alpha()
        );
}


void
image_invert::set_pixel(int x, int y, const icon_pixel &value)
{
    icon_pixel
        tmp
        (
            1. - value.get_red(),
            1. - value.get_green(),
            1. - value.get_blue(),
            value.get_alpha()
        );
    deeper->set_pixel(x, y, tmp);
}


unsigned
image_invert::get_width(void)
    const
{
    return deeper->get_width();
}


unsigned
image_invert::get_height(void)
    const
{
    return deeper->get_height();
}


void
image_invert::save_to_file(const std::string &filename)
    const
{
    deeper->save_to_file(filename);
}
