//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_INVERT_H
#define LIB_IMAGE_INVERT_H

#include <lib/image.h>

/**
  * The image_invert class is used to represent the processing required
  * to invert the colours of an image, pixel by pixel.
  */
class image_invert:
    public image
{
public:
    /**
      * The destructor.
      */
    virtual ~image_invert();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param deeper
      *     the source of the image to be inverted.
      */
    static pointer create(const image::pointer &deeper);

protected:
    // See base class for documentation.
    const char *type_name(void) const;

    // See base class for documentation.
    void get_pixel(int x, int y, icon_pixel &value) const;

    // See base class for documentation.
    void set_pixel(int x, int y, const icon_pixel &value);

    // See base class for documentation.
    unsigned get_width(void) const;

    // See base class for documentation.
    unsigned get_height(void) const;

    // See base class for documentation.
    void save_to_file(const std::string &filename) const;

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     the source of the image to be inverted.
      */
    image_invert(const image::pointer &deeper);

    /**
      * The deeper instance variable is used to remember the source of
      * the image to be inverted.
      */
    image::pointer deeper;

    /**
      * The default constructor.
      * It is private on purpose, use the #create class method instead.
      */
    image_invert();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    image_invert(const image_invert &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    image_invert &operator=(const image_invert &rhs);
};

#endif // LIB_IMAGE_INVERT_H
