//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_MONOCHROME_H
#define LIB_IMAGE_MONOCHROME_H

#include <lib/image.h>

/**
  * The image_monochrome class is used to represent a filter to turn a
  * color image into a monochrome image.
  */
class image_monochrome:
    public image
{
public:
    /**
      * The destructor.
      */
    virtual ~image_monochrome();

private:
    /**
      * The constructor.  It is private on purpose, use the "create"
      * class method instead.
      *
      * @param deeper
      *     The image to be filtered.
      */
    image_monochrome(const pointer &deeper);

protected:
    // See base class for documentation.
    const char *type_name() const;

    // See base class for documentation.
    void get_pixel(int x, int y, icon_pixel &value) const;

    // See base class for documentation.
    void set_pixel(int x, int y, const icon_pixel &value);

    // See base class for documentation.
    unsigned get_width() const;

    // See base class for documentation.
    unsigned get_height() const;

    // See base class for documentation.
    void save_to_file(const std::string &filename) const;

public:
    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param deeper
      *     The image to be filtered.
      */
    static pointer create(const pointer &deeper);

protected:

private:
    /**
      * The deeper instance variable is used to remember the image to be
      * filtered.
      */
    pointer deeper;

    /**
      * The default constructor.  Do not use.
      */
    image_monochrome();

    /**
      * The copy constructor.  Do not use.
      */
    image_monochrome(const image_monochrome &);

    /**
      * The assignment operator.  Do not use.
      */
    image_monochrome &operator=(const image_monochrome &);
};

#endif // LIB_IMAGE_MONOCHROME_H
