//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/image/threshold.h>


image_threshold::~image_threshold()
{
}


image_threshold::image_threshold(
    const image::pointer &a_deeper,
    double a_level
) :
    deeper(a_deeper),
    level(a_level)
{
}


image_threshold::pointer
image_threshold::create(const image::pointer &a_deeper, double a_level)
{
    return pointer(new image_threshold(a_deeper, a_level));
}


unsigned
image_threshold::get_width(void)
    const
{
    return deeper->get_width();
}


unsigned
image_threshold::get_height(void)
    const
{
    return deeper->get_height();
}


void
image_threshold::save_to_file(const std::string &filename)
    const
{
    deeper->save_to_file(filename);
}


const char *
image_threshold::type_name(void)
    const
{
    return deeper->type_name();
}


void
image_threshold::get_pixel(int x, int y, icon_pixel &p)
    const
{
    icon_pixel p2;
    deeper->get_pixel(x, y, p2);
    if (p2.get_intensity() >= level)
        p = icon_pixel(1., 1., 1., p2.get_alpha());
    else
        p = icon_pixel(0., 0., 0., p2.get_alpha());
}


void
image_threshold::set_pixel(int x, int y, const icon_pixel &p)
{
    deeper->set_pixel(x, y, p);
}
