//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_IMAGE_THRESHOLD_H
#define LIB_IMAGE_THRESHOLD_H

#include <lib/image.h>

/**
  * The image_threshold class is used to represent a filter that
  * thresholds and image avainst a certain pixel value.
  */
class image_threshold:
    public image
{
public:
    typedef boost::shared_ptr<image_threshold> pointer;

    /**
      * The destructor.
      */
    virtual ~image_threshold();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param deeper
      *     The source of image data.
      * @param level
      *     The amount of intensity of interest
      */
    static pointer create(const image::pointer &deeper, double level);

protected:
    // See base class for documentation.
    const char *type_name(void) const;

    // See base class for documentation.
    void get_pixel(int, int, icon_pixel&) const;

    // See base class for documentation.
    void set_pixel(int, int, const icon_pixel &);

    // See base class for documentation.
    unsigned int get_width(void) const;

    // See base class for documentation.
    unsigned int get_height(void) const;

    // See base class for documentation.
    void save_to_file(const std::string &) const;

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     The source of image data.
      * @param level
      *     The amount of intensity of interest
      */
    image_threshold(const image::pointer &deeper, double level);

    /**
      * The deeper instance variable is used to remember the source of
      * data to be filtered.
      */
    image::pointer deeper;

    /**
      * The level instance variable is used to remember the amount of
      * intensity of interest.
      */
    double level;

    /**
      * The default constructor.  Do not use.
      */
    image_threshold();

    /**
      * The copy constructor.  Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    image_threshold(const image_threshold &rhs);

    /**
      * The assignment operator.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    image_threshold &operator=(const image_threshold &rhs);
};

#endif // LIB_IMAGE_THRESHOLD_H
