//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_INTERPOSE_GOTO_H
#define LIB_INTERPOSE_GOTO_H

#include <lib/interpose.h>

/**
  * The interpose_goto class is used to represent
  */
class interpose_goto:
    public interpose
{
public:
    /**
      * The destructor.
      */
    virtual ~interpose_goto();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param deeper
      *     Where to send the stateless key stream.
      */
    static pointer create(const pointer &deeper);

protected:
    // See base class for documentation.
    void on_opcode(opcode_t op, const location::pointer &where);

    // See base class for documentation.
    bool strict_memory_number_checking_required(void) const;

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     Where to send the stateless key stream.
      */
    interpose_goto(const pointer &deeper);

    /**
      * The deeper instance variable is used to remember where to send
      * the stateless key stream.
      */
    pointer deeper;

    /**
      * The default constructor.
      * Do not use.
      */
    interpose_goto();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    interpose_goto(const interpose_goto &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    interpose_goto &operator=(const interpose_goto &rhs);
};

#endif // LIB_INTERPOSE_GOTO_H
