//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_INTERPOSE_MEMORY_H
#define LIB_INTERPOSE_MEMORY_H

#include <lib/interpose.h>

/**
  * The interpose_memory class is used to represent the processing
  * required to glue a key stroke to an xM3~ key, to build a complete
  * xM(n) pseudo keystroke.
  */
class interpose_memory:
    public interpose
{
public:
    /**
      * The destructor.
      */
    virtual ~interpose_memory();

    // See base class for documentation.
    void on_opcode(opcode_t op, const location::pointer &where);

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

protected:
    /**
      * The constructor.
      * For use by derived classes only.
      *
      * @param deeper
      *     Where to send the stateless key stream.
      * @param base
      *     The base value of the range of opcodes corresponding to
      *     digits, counting from zero (even if zero isn't legal).
      * @param start
      *     where the XM3~ token was
      */
    interpose_memory(const pointer &deeper, int base,
        const location::pointer &start);

    // See base class for documentation.
    bool strict_memory_number_checking_required(void) const;

private:
    /**
      * The deeper instance variable is used to remember
      * where to send the stateless key stream.
      */
    pointer deeper;

    /**
      * The base instance variable is used to remember the base value of
      * the range of opcodes corresponding to digits, counting from zero
      * (even if zero isn't legal).
      */
    int base;

    /**
      * The location of the first opcode, the one that is being extended.
      */
    location::pointer start;

    /**
      * The default constructor.
      * Do not use.
      */
    interpose_memory();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    interpose_memory(const interpose_memory &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    interpose_memory &operator=(const interpose_memory &rhs);
};

#endif // LIB_INTERPOSE_MEMORY_H
