//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_INTERPOSE_MEMORY_SM_H
#define LIB_INTERPOSE_MEMORY_SM_H

#include <lib/interpose/memory.h>

/**
  * The interpose_memory_sm class is used to represent
  * processing of the SM3~ {n} sequence into a single opcode.
  */
class interpose_memory_sm:
    public interpose_memory
{
public:
    /**
      * The destructor.
      */
    virtual ~interpose_memory_sm();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param deeper
      *     where to write the stateless key stream
      * @param start
      *     where the introducer opcode was seen
      */
    static pointer create(const pointer &deeper,
        const location::pointer &start);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param deeper
      *     where to write the stateless key stream
      * @param start
      *     where the introducer opcode was seen
      */
    interpose_memory_sm(const pointer &deeper,
        const location::pointer &start);

    /**
      * The default constructor.
      * Do not use.
      */
    interpose_memory_sm();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    interpose_memory_sm(const interpose_memory_sm &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    interpose_memory_sm &operator=(const interpose_memory_sm &rhs);
};

#endif // LIB_INTERPOSE_MEMORY_SM_H
