//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_INTERPOSE_PRO_CHE_H
#define LIB_INTERPOSE_PRO_CHE_H

#include <lib/interpose.h>

class calculator; // forward

/**
  * The interpose_pro_che class is used to represent sending keystrokes to a
  * calculator instance, when in Program Mode PRO CHE state.
  */
class interpose_pro_che:
    public interpose
{
public:
    /**
      * The destructor.
      */
    virtual ~interpose_pro_che();

    /**
      * The create class method is used to create new dynamically
      * allocated istances of this class.
      *
      * @param subject
      *     The calculator instance to drive.
      */
    static pointer create(calculator *subject);

protected:
    // See base class for documentation.
    void on_opcode(opcode_t op, const location::pointer &where);

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

    // See base class for documentation.
    bool strict_memory_number_checking_required(void) const;

private:
    /**
      * The constructor.
      *
      * @param subject
      *     The calculator instance to drive.
      */
    interpose_pro_che(calculator *subject);

    /**
      * The subject instance variable is used to remember the
      * calculator we are to send keystrokes to.
      */
    calculator *subject;

    /**
      * The default constructor.
      * Do not use.
      */
    interpose_pro_che();

    /**
      * The copy constructor.
      * Do not use.
      */
    interpose_pro_che(const interpose_pro_che &);

    /**
      * The assignment operator.
      * Do not use.
      */
    interpose_pro_che &operator=(const interpose_pro_che &);
};

#endif // LIB_INTERPOSE_PRO_CHE_H
