//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_INTERPOSE_STATEFUL_H
#define LIB_INTERPOSE_STATEFUL_H

#include <lib/interpose.h>

/**
  * The interpose_stateful class is used to represent the processing to
  * translate a stateful key stream into a stateless key stream.
  */
class interpose_stateful:
    public interpose
{
public:
    /**
      * The destructor.
      */
    virtual ~interpose_stateful();

    /**
      * The create class method is used to create new dynamically
      * allocated instances of this class.
      *
      * @param deeper
      *     Where to send the stateless key stream.
      * @param jumper
      *     whether or not to treat "UJ" statefully to become
      *     opcode_extended_goto.
      */
    static pointer create(const interpose::pointer &deeper,
        bool jumper = false);

protected:
    // See base class for documentation.
    void on_opcode(opcode_t op, const location::pointer &where);

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

    // See base class for documentation.
    bool strict_memory_number_checking_required(void) const;

private:
    /**
      * The constructor.
      *
      * @param deeper
      *     Where to send the stateless key stream.
      * @param jumper
      *     whether or not to treat "UJ" statefully to become
      *     opcode_extended_goto.
      */
    interpose_stateful(const interpose::pointer &deeper, bool jumper);

    /**
      * The deeper instance variable is used to remember where to send
      * the stateless key stream.
      */
    pointer deeper;

    /**
      * The jumper instance variuable is used to remember whether or not
      * to treat "UJ" statefully to become opcode_extended_goto.
      */
    bool jumper;

    /**
      * The diversion instance variable is used to remember where to
      * send the stateless key stream, when multiple keys need to be
      * folded into one virtual key.
      */
    pointer diversion;

    /**
      * The default constructor.
      * Do not use.
      */
    interpose_stateful();

    /**
      * The copy constructor.
      * Do not use.
      */
    interpose_stateful(const interpose_stateful &);

    /**
      * The assignment operator.
      * Do not use.
      */
    interpose_stateful &operator=(const interpose_stateful &);
};

#endif // LIB_INTERPOSE_STATEFUL_H
