//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_LEXER_FUNCTOR_H
#define LIB_LEXER_FUNCTOR_H

#include <boost/shared_ptr.hpp>

#include <lib/location.h>

/**
  * The lexer_functor class is used to represent an arbitrary action to
  * be performed when a keyword is seen by the #lexer.
  */
class lexer_functor
{
public:
    typedef boost::shared_ptr<lexer_functor> pointer;

    /**
      * The destructor.
      */
    virtual ~lexer_functor();

    /**
      * The function call operator, with a location argument, is to make this
      * object look like a function, that's why it's called a functor.
      */
    void operator()(const location::pointer &where) { do_it(where); }

protected:
    /**
      * The do_it method is provided by each derived class, to actually
      * perform the actions of the functor.
      */
    virtual void do_it(const location::pointer &where) = 0;

    /**
      * The default constructor.
      * For use by derived classes only.
      */
    lexer_functor();

private:
    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    lexer_functor(const lexer_functor &rhs);

public:
    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    lexer_functor &operator=(const lexer_functor &rhs);
};

#endif // LIB_LEXER_FUNCTOR_H
