//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_LEXER_FUNCTOR_METHOD_BOOL_H
#define LIB_LEXER_FUNCTOR_METHOD_BOOL_H

#include <lib/lexer/functor.h>

class calculator; // forward

/**
  * The lexer_functor_method_bool class is used to represent the processing
  * necessary to call an arbitrarty method of the calculator in response
  * to a keyword being observed in the lexer.
  */
class lexer_functor_method_bool:
    public lexer_functor
{
public:
    typedef boost::shared_ptr<lexer_functor_method_bool> pointer;

    /**
      * The destructor.
      */
    virtual ~lexer_functor_method_bool();

    typedef void (calculator::*method_ptr_t)(bool);

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param subject
      *     The calculator object to operate upon.
      * @param method
      *     The calculator method to be called.
      * @param value
      *     the boolean value to pass to the method.
      */
    static pointer create(calculator &subject, method_ptr_t method, bool value);

    // See base class for documentation.
    void do_it(const location::pointer &where);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param subject
      *     The calculator object to operate upon.
      * @param method
      *     The calculator method to be called.
      * @param value
      *     the boolean value to pass to the method.
      */
    lexer_functor_method_bool(calculator &subject, method_ptr_t method,
        bool value);

    /**
      * The subject instance variable is used to remember the calculator
      * object to operate upon.
      */
    calculator &subject;

    /**
      * The method instance variable is used to remember the calculator
      * method to be called.
      */
    method_ptr_t method;

    /**
      * The value instance variable is used to remember
      * the boolean value to pass to the method.
      */
    bool value;

    /**
      * The default constructor.
      * Do not use.
      */
    lexer_functor_method_bool();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    lexer_functor_method_bool(const lexer_functor_method_bool &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    lexer_functor_method_bool &operator=(const lexer_functor_method_bool &rhs);
};

#endif // LIB_LEXER_FUNCTOR_METHOD_BOOL_H
