//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_LEXER_FUNCTOR_OPCODE_H
#define LIB_LEXER_FUNCTOR_OPCODE_H

#include <lib/lexer/functor.h>
#include <lib/opcode.h>

class calculator; // forward

/**
  * The lexer_functor_opcode class is used to represent the processing
  * required when the lexer sees a keyword that matches an opcode name.
  * It is passed to the calculator::on_opcode method.
  */
class lexer_functor_opcode:
    public lexer_functor
{
public:
    typedef boost::shared_ptr<lexer_functor_opcode> pointer;

    /**
      * The destructor.
      */
    virtual ~lexer_functor_opcode();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param subject
      *     The calculator object on which to operate.
      * @param op
      *     The opcode the claculator is  to perform.
      */
    static pointer create(calculator &subject, opcode_t op);

    // See base class for documentation.
    void do_it(const location::pointer &where);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param subject
      *     The calculator object on which to operate.
      * @param op
      *     The opcode the calculator is  to perform.
      */
    lexer_functor_opcode(calculator &subject, opcode_t op);

    /**
      * The subject instance variable is used to remember the calculator
      * object we are to operate upon.
      */
    calculator &subject;

    /**
      * The op instance variable is used to remember the opcode the
      * calculator is to perform.
      */
    opcode_t op;

    /**
      * The default constructor.
      * Do not use.
      */
    lexer_functor_opcode();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    lexer_functor_opcode(const lexer_functor_opcode &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    lexer_functor_opcode &operator=(const lexer_functor_opcode &rhs);
};

#endif // LIB_LEXER_FUNCTOR_OPCODE_H
