//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_LEXER_FUNCTOR_PROGRAM_MODE_H
#define LIB_LEXER_FUNCTOR_PROGRAM_MODE_H

#include <lib/calculator.h>
#include <lib/lexer/functor.h>

/**
  * The lexer_functor_program_mode class is used to represent sending
  * program mode setting from keywords seen by the lexer.
  */
class lexer_functor_program_mode:
    public lexer_functor
{
public:
    typedef boost::shared_ptr<lexer_functor_program_mode> pointer;

    /**
      * The destructor.
      */
    virtual ~lexer_functor_program_mode();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      *
      * @param subject
      *     The claculkator object to operate upon.
      * @param program_mode
      *     The program mode to switched to.
      */
    static pointer create(calculator &subject,
        calculator::program_mode_t program_mode);

    // See base class for documentation.
    void do_it(const location::pointer &where);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param subject
      *     The calculator object to operate upon.
      * @param program_mode
      *     The program mode to switched to.
      */
    lexer_functor_program_mode(calculator &subject,
        calculator::program_mode_t program_mode);

    /**
      * The subject instance variable is used to remember the calculator
      * object to operate upon.
      */
    calculator &subject;

    /**
      * The fubar instance variable is used to remember the program mode
      * to switched to.
      */
    calculator::program_mode_t program_mode;
    /**
      * The default constructor.
      * Do not use.
      */
    lexer_functor_program_mode();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    lexer_functor_program_mode(const lexer_functor_program_mode &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    lexer_functor_program_mode &operator=(
        const lexer_functor_program_mode &rhs);
};

#endif // LIB_LEXER_FUNCTOR_PROGRAM_MODE_H
