//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/location.h>
#include <lib/std_string_printf.h>


location::~location()
{
}


location::location(
    const std::string &a_filename,
    int a_line_number,
    size_t a_byte_start,
    size_t a_byte_length
) :
    filename(a_filename),
    line_number(a_line_number),
    byte_start(a_byte_start),
    byte_length(a_byte_length)
{
}


location::pointer
location::create(const std::string &a_filename, int a_line_number,
    size_t a_byte_start, size_t a_byte_length)
{
    return
        pointer
        (
            new location(a_filename, a_line_number, a_byte_start, a_byte_length)
        );
}


location::pointer
location::span(const location::pointer &from, const location::pointer &to)
{
    if (!from)
        return to;
    if (!to)
        return from;
    return
        create
        (
            from->filename,
            from->line_number,
            from->byte_start,
            to->byte_start + to->byte_length - from->byte_start
        );
}


std::string
location::representation(void)
    const
{
    return
        std_string_printf
        (
            "{ filename = \"%s\", line_number = %d, byte_start = %zd, "
                "byte_length = %zd }",
            filename.c_str(),
            line_number,
            byte_start,
            byte_length
        );
}
