//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>
#include <lib/ac/math.h>

#include <lib/number.h>


void
number::operator/=(const number &rhs)
{
    *this = operator/(rhs);
}


number
number::operator/(const number &rhs)
    const
{
    if (rhs.is_zero())
        return number::infinity();

    bool neg = (negative != rhs.negative);

    unsigned t3ndec = 16;
    assert(t3ndec * 2 >= decimal);
    assert(t3ndec >= rhs.decimal);
    number_z t1(value.shift_left(t3ndec * 2 - decimal));
    number_z t2(rhs.value.shift_left(t3ndec - rhs.decimal));
    number_z t3 = t1 / t2;

    // Instruction Manual, p. 21:
    // "If the number of digits in the result exceeds 16, the leftmost
    // digit priority system functions and excess digits at the right
    // are dropped."
    truncate_floating_operation(t3, t3ndec);

    // <question>
    // Does division strip redundant decimal places to the right of the
    // decimal point?
    // </question>
    strip_redundant_decimal_places(t3, t3ndec);

    return number(t3, t3ndec, neg);
}


// vim: set ts=8 sw=4 et :
