//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


display
number::get_display(void)
    const
{
    display result;

    int dec_pt = decimal;
    bool zero_sup = true;
    if (has_overflowed())
    {
        // Instruction Manual, p. 21:
        //
        // "If the number of integral digits exceeds 16, the position
        // of the decimal point automatically indicates the number of
        // integral digits that have been dropped.  (For instance, if 3
        // integral digits are dropped, the decimal point appears after
        // the third digit from the left.)  In this case, the overflow
        // lamp lights, but the actual position of the decimal point is
        // preserved within the registers."
        //
        // Note: from the LEFT
        //
        result.overflow = true;
        assert(decimal == 0);
        size_t vdc = value.get_digit_count();
        if (vdc >= 32)
            dec_pt = 0;
        else
            dec_pt = 32 - vdc;

        // <question>
        // Is zero supression active for overflowed values?  It probably
        // should not be, cponsidering there are valid digits to the
        // left of those displayed.
        // </question>
        zero_sup = false;
    }

    for (int dignum = 15; dignum >= 0; --dignum)
    {
        int n = value.get(dignum);
        if ((unsigned)dignum <= dec_pt)
            zero_sup = false;
        if (n != 0)
            zero_sup = false;
        display::tube_t &tube = result.tubes[dignum];
        if (n != 0 || !zero_sup)
            tube.digit = n;
        else
            tube.digit = 10;
        tube.dot = ((unsigned)dignum == dec_pt);
        tube.tick = (dignum > 0) && (((18 + dignum - decimal) % 3) == 0);
    }
    result.negative = negative;
    return result;
}


// vim: set ts=8 sw=4 et :
