//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


void
number::operator*=(const number &rhs)
{
    *this = operator*(rhs);
}


number
number::operator*(const number &rhs)
    const
{
    bool neg = (negative != rhs.negative);
    number_z t1(value);
    number_z t2(rhs.value);
    number_z t3(t1 * t2);
    unsigned t3ndec = decimal + rhs.decimal;

    // Instruction Manual, p. 21:
    // "If the number of digits in the result exceeds 16, the leftmost
    // digit priority system functions and excess digits at the right
    // are dropped."
    truncate_floating_operation(t3, t3ndec);
    assert(t3ndec < 16);

    // <question>
    // Does multiply strip redundant decimal places to the right of the
    // decimal point?
    // Does "1.25 [times] 1.22 [+=]" display "1.525" or "1.5250"?
    // </question>
#if 1
    strip_redundant_decimal_places(t3, t3ndec);
#endif

    return number(t3, t3ndec, neg);
}


// vim: set ts=8 sw=4 et :
