//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


number
number::positive_minus_positive(const number &lhs, const number &rhs)
{
    assert(!lhs.negative);
    assert(!rhs.negative);

    // short circuit the easy case
    if (rhs.value.is_zero())
        return lhs;

    // Note: overflow cannot occur, because the abolute value of the result
    // will always be less than max(t1, t2), and since neither t1 nor t2 are
    // overflow, and both are positive, no overflow can result.

    unsigned ndec = max(lhs.decimal, rhs.decimal);
    number_z t1(lhs.value);
    if (ndec > lhs.decimal)
        t1 = t1.shift_left(ndec - lhs.decimal);
    number_z t2(rhs.value);
    if (ndec > rhs.decimal)
        t2 = t2.shift_left(ndec - rhs.decimal);
    if (t1 >= t2)
    {
        number_z t3 = t1 - t2;
        truncate_floating_operation(t3, ndec);
        return number(t3, ndec, false);
    }
    else
    {
        number_z t3 = t2 - t1;
        truncate_floating_operation(t3, ndec);
        return number(t3, ndec, true);
    }
}


// vim: set ts=8 sw=4 et :
