//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


number
number::positive_plus_positive(const number &lhs, const number &rhs)
{
    assert(!lhs.negative);
    assert(!rhs.negative);

    // short circuit the easy cases
    if (lhs.value.is_zero())
        return rhs;
    if (rhs.value.is_zero())
        return lhs;

    // In this case, both lhs and rhs are positive, meaning we
    // can do it with Z arithmetic alone.
    number_z t1(lhs.value);
    number_z t2(rhs.value);
    unsigned t3dec = max(lhs.decimal, rhs.decimal);
    if (t3dec > lhs.decimal)
        t1 = t1.shift_left(t3dec - lhs.decimal);
    if (t3dec > rhs.decimal)
        t2 = t2.shift_left(t3dec - rhs.decimal);
    number_z t3(t1 + t2);

    // Instruction Manual, p. 21:
    // "If the number of digits in the result exceeds 16, the leftmost
    // digit priority system functions and excess digits at the right
    // are dropped."
    truncate_floating_operation(t3, t3dec);

    return number(t3, t3dec);
}


// vim: set ts=8 sw=4 et :
