//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


void
number::right_shift(void)
{
    // <question>
    // When you have an overflow on the screen (e.g. divide by zero,
    // 9999999999999999) when you press [->] what is displayed?
    // Is it "0.", "999999999999999.", or something else?
    // </question>

    // Instruction Manual, p. 21:
    //
    // "If the number of integral digits exceeds 16, the position of
    // the decimal point automatically indicates the number of integral
    // digits that have been dropped.  (For instance, if 3 integral
    // digits are dropped, the decimal point appears after the third
    // digit from the left.)  In this case, the overflow lamp lights,
    // but the actual position of the decimal point is preserved within
    // the registers."
    //
    // Note: from the LEFT.
    //
    if (has_overflowed())
    {
        // <question>
        // When a calculation overflows, an you press [->] to clear it,
        // what happens to the display?  Is the overflow light cleared,
        // but the value on the screen changes?  Does the value on the
        // screen becoem 15 digits, with a leading 0 digit?  Does the
        // 17th digit get shifted in?
        // </question>
        //
        // This code keeps the right-most 16 digits, and discards the rest.
        // A fairly literal interpretation of clearing the overflow indication.
        //
        assert(decimal == 0);
        value = value.right_16();
    }
    else
    {
        value = value.shift_right(1);
        if (decimal > 0)
            --decimal;
    }
    if (value.is_zero())
        negative = false;
}


// vim: set ts=8 sw=4 et :
