//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>


//
// Instruction Manual: p. 21
//
// "If the number of digits in the result exceeds 16, the leftmost
// digit priority system functions and excess digits at the right are
// dropped."
//
// I.e. intermediate results round down.
//
#undef ROUND_OFF


void
number::truncate_floating_operation(number_z &t3, unsigned &t3ndec)
{
    size_t t3dc = t3.get_digit_count();
    if (t3dc > 16 || t3ndec > 15)
    {
        //
        // Truncate decimal places to the right of the decimal point.
        //
        // Instruction Manual, p. 21:
        //
        // "If the number of digits in the result exceeds 16, the
        // leftmost digit priority system functions and excess digits at
        // the right are dropped."
        //
        size_t dump_this_many = 0;
        if (t3dc > 16)
        {
            dump_this_many = t3dc - 16;
            if (dump_this_many > t3ndec)
                dump_this_many = t3ndec;
        }
        if (t3ndec - dump_this_many > 15)
            dump_this_many = t3ndec - 15;
#ifdef ROUND_OFF
        bool nudge = (dump_this_many > 0) && (t3.get(dump_this_many - 1) >= 5);
#endif
        t3ndec -= dump_this_many;
        t3 = t3.shift_right(dump_this_many);
#ifdef ROUND_OFF
        if (nudge)
            t3 += number_z::one();
#endif
    }
}


// vim: set ts=8 sw=4 et :
