//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number/z.h>


number_z &
number_z::operator+=(const number_z &rhs)
{
    assert(is_valid());
    assert(rhs.is_valid());
    unsigned carry = 0;
    for (size_t n = 0; carry || n < digits_used || n < rhs.digits_used; ++n)
    {
        unsigned dsum = get(n) + rhs.get(n) + carry;
        carry = dsum / 10;
        set(n, dsum % 10);
    }
    assert(is_valid());
    return *this;
}


number_z
number_z::operator+(const number_z &rhs)
    const
{
    number_z result(*this);
    result += rhs;
    return result;
}


// vim: set ts=8 sw=4 et :
