//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>
#include <lib/ac/string.h>

#include <lib/number/z.h>


number_z::number_z(unsigned rhs) :
    digits_used(0),
    digits_allocated(0),
    digits(0)
{
    if (rhs)
    {
        typeof(rhs) n = rhs;
        for (;;)
        {
            ++digits_allocated;
            n /= 10;
            if (!n)
                break;
        }
        for (;;)
        {
            size_t lsb = (digits_allocated & -digits_allocated);
            if (lsb == digits_allocated)
                break;
            digits_allocated += lsb;
        }

        digits = new digit_t [digits_allocated];
        for (;;)
        {
            digits[digits_used] = rhs % 10;
            ++digits_used;
            rhs /= 10;
            if (!rhs)
                break;
        }
        memset(digits + digits_used, 0, digits_allocated - digits_used);
    }
    assert(is_valid());
}


// vim: set ts=8 sw=4 et :
