//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number/z.h>


number_z &
number_z::operator*=(const number_z &rhs)
{
    *this = *this * rhs;
    return *this;
}


number_z
number_z::operator*(const number_z &rhs)
    const
{
    assert(is_valid());
    assert(rhs.is_valid());

    // short circuit the easy cases
    if (is_zero() || rhs.is_zero())
        return number_z();
    if (is_one())
        return rhs;
    if (rhs.is_one())
        return *this;

    number_z result;
    for (size_t j = 0; j < digits_used; ++j)
    {
        unsigned carry = 0;
        for (size_t k = 0; k < rhs.digits_used || carry; ++k)
        {
            unsigned dsum =
                result.get(j + k) + digits[j] * rhs.get(k) + carry;
            carry = dsum / 10;
            result.set(j + k, dsum % 10);
        }
    }
    return result;
}


// vim: set ts=8 sw=4 et :
