//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>
#include <lib/ac/limits.h>
#include <lib/ac/string.h>

#include <lib/number/z.h>


void
number_z::set(size_t n, unsigned value)
{
    assert(is_valid());
    assert(value < 10);
    assert(n < INT_MAX);
    if (value == 0 && n >= digits_used)
        return;
    if (n + 1 > digits_allocated)
    {
        size_t new_digits_allocated =
            (digits_allocated ? (digits_allocated * 2) : 16);
        for (;;)
        {
            if (n + 1 <= new_digits_allocated)
                break;
            new_digits_allocated *= 2;
        }
        digit_t *new_digits = new digit_t [new_digits_allocated];
        memcpy(new_digits, digits, digits_used);
        memset(new_digits + digits_used, 0, new_digits_allocated - digits_used);
        delete [] digits;
        digits = new_digits;
        digits_allocated = new_digits_allocated;
    }
    assert(n + 1 <= digits_allocated);
    digits[n] = value;
    if (n >= digits_used)
        digits_used = n + 1;

    // If they set the most significant digit to zero, we must normalize.
    // (The assert right at the start of this method ensured that the most
    // significant digit was not zero when we entered this method, so the loop
    // body should be the uncommon case.)
    while (digits_used > 0 && digits[digits_used - 1] == 0)
        --digits_used;

    assert(is_valid());
}


// vim: set ts=8 sw=4 et :
