//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/opcode.h>
#include <lib/std_string_printf.h>
#include <lib/unicode.h>


std::string
opcode_name(opcode_t n)
{
    switch (n)
    {
    case opcode_zero:
        goto yuck;

    case opcode_plus_equals:
        return "+=";

    case opcode_minus_equals:
        return "-=";

    case opcode_mul:
        return "*";

    case opcode_div:
        return "/";

    case opcode_sqrt:
        return "SQRT";

    case opcode_ent:
        return "ENT";

    case opcode_sj:
        return "SJ";

    case opcode_ej:
        return "EJ";

    case opcode_mj:
        return "MJ";

    case opcode_uj:
        return "UJ";

    case opcode_fj:
        return "FJ";

    case opcode_suj:
        return "SUJ";

    case opcode_sfj:
        return "SFJ";

    case opcode_srj:
        return "SRJ";

    case opcode_m1:
        return "M1";

    case opcode_m2:
        return "M2";

    case opcode_m3:
        return "M3";

    case opcode_m4:
        return "M4";

    case opcode_m5:
        return "M5";

    case opcode_m6:
        return "M6";

    case opcode_m7:
        return "M7";

    case opcode_m8:
        return "M8";

    case opcode_m9:
        return "M9";

    case opcode_m10:
        return "M10";

    case opcode_m11:
        return "M11";

    case opcode_m12:
        return "M12";

    case opcode_m13:
        return "M13";

    case opcode_m14:
        return "M14";

    case opcode_mm1:
        return "MM1";

    case opcode_mm2:
        return "MM2";

    case opcode_sm3:
        return "SM3";

    case opcode_sm4:
        return "SM4";

    case opcode_sm5:
        return "SM5";

    case opcode_sm6:
        return "SM6";

    case opcode_sm7:
        return "SM7";

    case opcode_sm8:
        return "SM8";

    case opcode_sm9:
        return "SM9";

    case opcode_sm10:
        return "SM10";

    case opcode_sm11:
        return "SM11";

    case opcode_sm12:
        return "SM12";

    case opcode_sm13:
        return "SM13";

    case opcode_sm14:
        return "SM14";

    case opcode_rm1:
        return "RM1";

    case opcode_rm2:
        return "RM2";

    case opcode_rm3:
        return "RM3";

    case opcode_rm4:
        return "RM4";

    case opcode_rm5:
        return "RM5";

    case opcode_rm6:
        return "RM6";

    case opcode_rm7:
        return "RM7";

    case opcode_rm8:
        return "RM8";

    case opcode_rm9:
        return "RM9";

    case opcode_rm10:
        return "RM10";

    case opcode_rm11:
        return "RM11";

    case opcode_rm12:
        return "RM12";

    case opcode_rm13:
        return "RM13";

    case opcode_rm14:
        return "RM14";

    case opcode_clear_indicator:
        return "CI";

    case opcode_cm1:
        return "CM1";

    case opcode_cm2:
        return "CM2";

    case opcode_cm3:
        return "CM3";

    case opcode_cm4:
        return "CM4";

    case opcode_cm5:
        return "CM5";

    case opcode_cm6:
        return "CM6";

    case opcode_cm7:
        return "CM7";

    case opcode_cm8:
        return "CM8";

    case opcode_cm9:
        return "CM9";

    case opcode_cm10:
        return "CM10";

    case opcode_cm11:
        return "CM11";

    case opcode_cm12:
        return "CM12";

    case opcode_cm13:
        return "CM13";

    case opcode_cm14:
        return "CM14";

    case opcode_dot:
        return ".";

    case opcode_rv:
        return "RV";

    case opcode_right:
        return "->";

    case opcode_chg_sign:
        return "CS";

    case opcode_round_down:
        return "FLOOR";

    case opcode_round_off:
        return "ROUND";

    case opcode_round_up:
        return "CEIL";

    case opcode_print:
        return "PRINT";

    case opcode_fd:
        return "FD";

    case opcode_n0:
        return "0";

    case opcode_n1:
        return "1";

    case opcode_n2:
        return "2";

    case opcode_n3:
        return "3";

    case opcode_n4:
        return "4";

    case opcode_n5:
        return "5";

    case opcode_n6:
        return "6";

    case opcode_n7:
        return "7";

    case opcode_n8:
        return "8";

    case opcode_n9:
        return "9";

    case opcode_n10:
        return "$710"; // otherwise is ambiguous

    case opcode_n11:
        return "$711"; // otherwise is ambiguous

    case opcode_n12:
        return "$712"; // otherwise is ambiguous

    case opcode_n13:
        return "$713"; // otherwise is ambiguous

    case opcode_n14:
        return "$714"; // otherwise is ambiguous

    case opcode_extended_c:
        return "C";

    case opcode_extended_start:
        return "START";

    case opcode_extended_m3_tilde:
        return "M3~";

    case opcode_extended_sm3_tilde:
        return "SM3~";

    case opcode_extended_rm3_tilde:
        return "RM3~";

    case opcode_extended_cm3_tilde:
        return "CM3~";

    case opcode_extended_off:
        return "OFF";

    case opcode_extended_program_print:
        return "PROGRAM_PRINT";

    case opcode_extended_display_print:
        return "DISPLAY_PRINT";

    case opcode_extended_paper_feed:
        return "PAPER_FEED";

    case opcode_extended_rounding_switch_up:
        return "rounding_switch_up";

    case opcode_extended_rounding_switch_off:
        return "rounding_switch_off";

    case opcode_extended_rounding_switch_down:
        return "rounding_switch_down";

    case opcode_extended_decimal_point_selector_0:
        return "decimal_point_selector_0";

    case opcode_extended_decimal_point_selector_1:
        return "decimal_point_selector_1";

    case opcode_extended_decimal_point_selector_2:
        return "decimal_point_selector_2";

    case opcode_extended_decimal_point_selector_3:
        return "decimal_point_selector_3";

    case opcode_extended_decimal_point_selector_4:
        return "decimal_point_selector_4";

    case opcode_extended_decimal_point_selector_5:
        return "decimal_point_selector_5";

    case opcode_extended_decimal_point_selector_6:
        return "decimal_point_selector_6";

    case opcode_extended_decimal_point_selector_7:
        return "decimal_point_selector_7";

    case opcode_extended_decimal_point_selector_8:
        return "decimal_point_selector_8";

    case opcode_extended_decimal_point_selector_9:
        return "decimal_point_selector_9";

    case opcode_extended_decimal_point_selector_10:
        return "decimal_point_selector_10";

    case opcode_extended_decimal_point_selector_11:
        return "decimal_point_selector_11";

    case opcode_extended_decimal_point_selector_12:
        return "decimal_point_selector_12";

    case opcode_extended_decimal_point_selector_float:
        return "decimal_point_selector_float";

    case opcode_extended_goto_000:
    case opcode_extended_goto_001:
    case opcode_extended_goto_002:
    case opcode_extended_goto_003:
    case opcode_extended_goto_004:
    case opcode_extended_goto_005:
    case opcode_extended_goto_006:
    case opcode_extended_goto_007:
    case opcode_extended_goto_008:
    case opcode_extended_goto_009:
    case opcode_extended_goto_010:
    case opcode_extended_goto_011:
    case opcode_extended_goto_012:
    case opcode_extended_goto_013:
    case opcode_extended_goto_014:
    case opcode_extended_goto_015:
    case opcode_extended_goto_100:
    case opcode_extended_goto_101:
    case opcode_extended_goto_102:
    case opcode_extended_goto_103:
    case opcode_extended_goto_104:
    case opcode_extended_goto_105:
    case opcode_extended_goto_106:
    case opcode_extended_goto_107:
    case opcode_extended_goto_108:
    case opcode_extended_goto_109:
    case opcode_extended_goto_110:
    case opcode_extended_goto_111:
    case opcode_extended_goto_112:
    case opcode_extended_goto_113:
    case opcode_extended_goto_114:
    case opcode_extended_goto_115:
    case opcode_extended_goto_200:
    case opcode_extended_goto_201:
    case opcode_extended_goto_202:
    case opcode_extended_goto_203:
    case opcode_extended_goto_204:
    case opcode_extended_goto_205:
    case opcode_extended_goto_206:
    case opcode_extended_goto_207:
    case opcode_extended_goto_208:
    case opcode_extended_goto_209:
    case opcode_extended_goto_210:
    case opcode_extended_goto_211:
    case opcode_extended_goto_212:
    case opcode_extended_goto_213:
    case opcode_extended_goto_214:
    case opcode_extended_goto_215:
    case opcode_extended_goto_300:
    case opcode_extended_goto_301:
    case opcode_extended_goto_302:
    case opcode_extended_goto_303:
    case opcode_extended_goto_304:
    case opcode_extended_goto_305:
    case opcode_extended_goto_306:
    case opcode_extended_goto_307:
    case opcode_extended_goto_308:
    case opcode_extended_goto_309:
    case opcode_extended_goto_310:
    case opcode_extended_goto_311:
    case opcode_extended_goto_312:
    case opcode_extended_goto_313:
    case opcode_extended_goto_314:
    case opcode_extended_goto_315:
    case opcode_extended_goto_400:
    case opcode_extended_goto_401:
    case opcode_extended_goto_402:
    case opcode_extended_goto_403:
    case opcode_extended_goto_404:
    case opcode_extended_goto_405:
    case opcode_extended_goto_406:
    case opcode_extended_goto_407:
    case opcode_extended_goto_408:
    case opcode_extended_goto_409:
    case opcode_extended_goto_410:
    case opcode_extended_goto_411:
    case opcode_extended_goto_412:
    case opcode_extended_goto_413:
    case opcode_extended_goto_414:
    case opcode_extended_goto_415:
    case opcode_extended_goto_500:
    case opcode_extended_goto_501:
    case opcode_extended_goto_502:
    case opcode_extended_goto_503:
    case opcode_extended_goto_504:
    case opcode_extended_goto_505:
    case opcode_extended_goto_506:
    case opcode_extended_goto_507:
    case opcode_extended_goto_508:
    case opcode_extended_goto_509:
    case opcode_extended_goto_510:
    case opcode_extended_goto_511:
    case opcode_extended_goto_512:
    case opcode_extended_goto_513:
    case opcode_extended_goto_514:
    case opcode_extended_goto_515:
    case opcode_extended_goto_600:
    case opcode_extended_goto_601:
    case opcode_extended_goto_602:
    case opcode_extended_goto_603:
    case opcode_extended_goto_604:
    case opcode_extended_goto_605:
    case opcode_extended_goto_606:
    case opcode_extended_goto_607:
    case opcode_extended_goto_608:
    case opcode_extended_goto_609:
    case opcode_extended_goto_610:
    case opcode_extended_goto_611:
    case opcode_extended_goto_612:
    case opcode_extended_goto_613:
    case opcode_extended_goto_614:
    case opcode_extended_goto_615:
    case opcode_extended_goto_700:
    case opcode_extended_goto_701:
    case opcode_extended_goto_702:
    case opcode_extended_goto_703:
    case opcode_extended_goto_704:
    case opcode_extended_goto_705:
    case opcode_extended_goto_706:
    case opcode_extended_goto_707:
    case opcode_extended_goto_708:
    case opcode_extended_goto_709:
    case opcode_extended_goto_710:
    case opcode_extended_goto_711:
    case opcode_extended_goto_712:
    case opcode_extended_goto_713:
    case opcode_extended_goto_714:
    case opcode_extended_goto_715:
        return ("UJ@" + opcode_name(opcode_t(n - opcode_extended_goto_000)));

    default:
        yuck:
        return opcode_raw_name(n);
    }
}


std::string
opcode_uname(opcode_t n)
{
    switch (n)
    {
    case opcode_zero:
        goto yuck;

    case opcode_plus_equals:
        return "+=";

    case opcode_minus_equals:
        return UNICODE_MINUS "=";

    case opcode_mul:
        return UNICODE_MUL;

    case opcode_div:
        return UNICODE_DIV;

    case opcode_sqrt:
        return UNICODE_SQRT;

    case opcode_ent:
        return "ENT";

    case opcode_sj:
        return "SJ";

    case opcode_ej:
        return "EJ";

    case opcode_mj:
        return "MJ";

    case opcode_uj:
        return "UJ";

    case opcode_fj:
        return "FJ";

    case opcode_suj:
        return "SUJ";

    case opcode_sfj:
        return "SFJ";

    case opcode_srj:
        return "SRJ";

    case opcode_m1:
        return "M1";

    case opcode_m2:
        return "M2";

    case opcode_m3:
        return "M3";

    case opcode_m4:
        return "M4";

    case opcode_m5:
        return "M5";

    case opcode_m6:
        return "M6";

    case opcode_m7:
        return "M7";

    case opcode_m8:
        return "M8";

    case opcode_m9:
        return "M9";

    case opcode_m10:
        return "M10";

    case opcode_m11:
        return "M11";

    case opcode_m12:
        return "M12";

    case opcode_m13:
        return "M13";

    case opcode_m14:
        return "M14";

    case opcode_mm1:
        return "MM1";

    case opcode_mm2:
        return "MM2";

    case opcode_sm3:
        return "SM3";

    case opcode_sm4:
        return "SM4";

    case opcode_sm5:
        return "SM5";

    case opcode_sm6:
        return "SM6";

    case opcode_sm7:
        return "SM7";

    case opcode_sm8:
        return "SM8";

    case opcode_sm9:
        return "SM9";

    case opcode_sm10:
        return "SM10";

    case opcode_sm11:
        return "SM11";

    case opcode_sm12:
        return "SM12";

    case opcode_sm13:
        return "SM13";

    case opcode_sm14:
        return "SM14";

    case opcode_rm1:
        return "RM1";

    case opcode_rm2:
        return "RM2";

    case opcode_rm3:
        return "RM3";

    case opcode_rm4:
        return "RM4";

    case opcode_rm5:
        return "RM5";

    case opcode_rm6:
        return "RM6";

    case opcode_rm7:
        return "RM7";

    case opcode_rm8:
        return "RM8";

    case opcode_rm9:
        return "RM9";

    case opcode_rm10:
        return "RM10";

    case opcode_rm11:
        return "RM11";

    case opcode_rm12:
        return "RM12";

    case opcode_rm13:
        return "RM13";

    case opcode_rm14:
        return "RM14";

    case opcode_clear_indicator:
        return "CE";

    case opcode_cm1:
        return "CM1";

    case opcode_cm2:
        return "CM2";

    case opcode_cm3:
        return "CM3";

    case opcode_cm4:
        return "CM4";

    case opcode_cm5:
        return "CM5";

    case opcode_cm6:
        return "CM6";

    case opcode_cm7:
        return "CM7";

    case opcode_cm8:
        return "CM8";

    case opcode_cm9:
        return "CM9";

    case opcode_cm10:
        return "CM10";

    case opcode_cm11:
        return "CM11";

    case opcode_cm12:
        return "CM12";

    case opcode_cm13:
        return "CM13";

    case opcode_cm14:
        return "CM14";

    case opcode_dot:
        return ".";

    case opcode_rv:
        return "RV";

    case opcode_right:
        return UNICODE_RIGHT;

    case opcode_chg_sign:
        return "CS";

    case opcode_round_down:
        return UNICODE_DOWN;

    case opcode_round_off:
        return "5/4";

    case opcode_round_up:
        return UNICODE_UP;

    case opcode_print:
        return UNICODE_PRINT;

    case opcode_fd:
        return "FD";

    case opcode_n0:
        return "0";

    case opcode_n1:
        return "1";

    case opcode_n2:
        return "2";

    case opcode_n3:
        return "3";

    case opcode_n4:
        return "4";

    case opcode_n5:
        return "5";

    case opcode_n6:
        return "6";

    case opcode_n7:
        return "7";

    case opcode_n8:
        return "8";

    case opcode_n9:
        return "9";

    case opcode_n10:
        return "10";

    case opcode_n11:
        return "11";

    case opcode_n12:
        return "12";

    case opcode_n13:
        return "13";

    case opcode_n14:
        return "14";

    case opcode_extended_c:
        return "C";

    case opcode_extended_start:
        return "START";

    case opcode_extended_m3_tilde:
        return "M3~";

    case opcode_extended_sm3_tilde:
        return "SM3~";

    case opcode_extended_rm3_tilde:
        return "RM3~";

    case opcode_extended_cm3_tilde:
        return "CM3~";

    case opcode_extended_off:
        return "OFF";

    case opcode_extended_program_print:
        return "PROGRAM_PRINT";

    case opcode_extended_display_print:
        return "DISPLAY_PRINT";

    case opcode_extended_paper_feed:
        return "PAPER_FEED";

    case opcode_extended_rounding_switch_up:
        return "rounding_switch_up";

    case opcode_extended_rounding_switch_off:
        return "rounding_switch_off";

    case opcode_extended_rounding_switch_down:
        return "rounding_switch_down";

    case opcode_extended_decimal_point_selector_0:
        return "decimal_point_selector_0";

    case opcode_extended_decimal_point_selector_1:
        return "decimal_point_selector_1";

    case opcode_extended_decimal_point_selector_2:
        return "decimal_point_selector_2";

    case opcode_extended_decimal_point_selector_3:
        return "decimal_point_selector_3";

    case opcode_extended_decimal_point_selector_4:
        return "decimal_point_selector_4";

    case opcode_extended_decimal_point_selector_5:
        return "decimal_point_selector_5";

    case opcode_extended_decimal_point_selector_6:
        return "decimal_point_selector_6";

    case opcode_extended_decimal_point_selector_7:
        return "decimal_point_selector_7";

    case opcode_extended_decimal_point_selector_8:
        return "decimal_point_selector_8";

    case opcode_extended_decimal_point_selector_9:
        return "decimal_point_selector_9";

    case opcode_extended_decimal_point_selector_10:
        return "decimal_point_selector_10";

    case opcode_extended_decimal_point_selector_11:
        return "decimal_point_selector_11";

    case opcode_extended_decimal_point_selector_12:
        return "decimal_point_selector_12";

    case opcode_extended_decimal_point_selector_float:
        return "decimal_point_selector_float";

    case opcode_extended_goto_000:
    case opcode_extended_goto_001:
    case opcode_extended_goto_002:
    case opcode_extended_goto_003:
    case opcode_extended_goto_004:
    case opcode_extended_goto_005:
    case opcode_extended_goto_006:
    case opcode_extended_goto_007:
    case opcode_extended_goto_008:
    case opcode_extended_goto_009:
    case opcode_extended_goto_010:
    case opcode_extended_goto_011:
    case opcode_extended_goto_012:
    case opcode_extended_goto_013:
    case opcode_extended_goto_014:
    case opcode_extended_goto_015:
    case opcode_extended_goto_100:
    case opcode_extended_goto_101:
    case opcode_extended_goto_102:
    case opcode_extended_goto_103:
    case opcode_extended_goto_104:
    case opcode_extended_goto_105:
    case opcode_extended_goto_106:
    case opcode_extended_goto_107:
    case opcode_extended_goto_108:
    case opcode_extended_goto_109:
    case opcode_extended_goto_110:
    case opcode_extended_goto_111:
    case opcode_extended_goto_112:
    case opcode_extended_goto_113:
    case opcode_extended_goto_114:
    case opcode_extended_goto_115:
    case opcode_extended_goto_200:
    case opcode_extended_goto_201:
    case opcode_extended_goto_202:
    case opcode_extended_goto_203:
    case opcode_extended_goto_204:
    case opcode_extended_goto_205:
    case opcode_extended_goto_206:
    case opcode_extended_goto_207:
    case opcode_extended_goto_208:
    case opcode_extended_goto_209:
    case opcode_extended_goto_210:
    case opcode_extended_goto_211:
    case opcode_extended_goto_212:
    case opcode_extended_goto_213:
    case opcode_extended_goto_214:
    case opcode_extended_goto_215:
    case opcode_extended_goto_300:
    case opcode_extended_goto_301:
    case opcode_extended_goto_302:
    case opcode_extended_goto_303:
    case opcode_extended_goto_304:
    case opcode_extended_goto_305:
    case opcode_extended_goto_306:
    case opcode_extended_goto_307:
    case opcode_extended_goto_308:
    case opcode_extended_goto_309:
    case opcode_extended_goto_310:
    case opcode_extended_goto_311:
    case opcode_extended_goto_312:
    case opcode_extended_goto_313:
    case opcode_extended_goto_314:
    case opcode_extended_goto_315:
    case opcode_extended_goto_400:
    case opcode_extended_goto_401:
    case opcode_extended_goto_402:
    case opcode_extended_goto_403:
    case opcode_extended_goto_404:
    case opcode_extended_goto_405:
    case opcode_extended_goto_406:
    case opcode_extended_goto_407:
    case opcode_extended_goto_408:
    case opcode_extended_goto_409:
    case opcode_extended_goto_410:
    case opcode_extended_goto_411:
    case opcode_extended_goto_412:
    case opcode_extended_goto_413:
    case opcode_extended_goto_414:
    case opcode_extended_goto_415:
    case opcode_extended_goto_500:
    case opcode_extended_goto_501:
    case opcode_extended_goto_502:
    case opcode_extended_goto_503:
    case opcode_extended_goto_504:
    case opcode_extended_goto_505:
    case opcode_extended_goto_506:
    case opcode_extended_goto_507:
    case opcode_extended_goto_508:
    case opcode_extended_goto_509:
    case opcode_extended_goto_510:
    case opcode_extended_goto_511:
    case opcode_extended_goto_512:
    case opcode_extended_goto_513:
    case opcode_extended_goto_514:
    case opcode_extended_goto_515:
    case opcode_extended_goto_600:
    case opcode_extended_goto_601:
    case opcode_extended_goto_602:
    case opcode_extended_goto_603:
    case opcode_extended_goto_604:
    case opcode_extended_goto_605:
    case opcode_extended_goto_606:
    case opcode_extended_goto_607:
    case opcode_extended_goto_608:
    case opcode_extended_goto_609:
    case opcode_extended_goto_610:
    case opcode_extended_goto_611:
    case opcode_extended_goto_612:
    case opcode_extended_goto_613:
    case opcode_extended_goto_614:
    case opcode_extended_goto_615:
    case opcode_extended_goto_700:
    case opcode_extended_goto_701:
    case opcode_extended_goto_702:
    case opcode_extended_goto_703:
    case opcode_extended_goto_704:
    case opcode_extended_goto_705:
    case opcode_extended_goto_706:
    case opcode_extended_goto_707:
    case opcode_extended_goto_708:
    case opcode_extended_goto_709:
    case opcode_extended_goto_710:
    case opcode_extended_goto_711:
    case opcode_extended_goto_712:
    case opcode_extended_goto_713:
    case opcode_extended_goto_714:
    case opcode_extended_goto_715:
        return "UJ@" + opcode_raw_name(opcode_t(n - opcode_extended_goto_000));

    default:
        yuck:
        return opcode_raw_name(n);
    }
}


std::string
opcode_raw_name(opcode_t n)
{
    if (n < 0 || n >= 128)
        return std_string_printf("UNKNOWN_%d", n);
    return std_string_printf("$%d%02d", ((n >> 4) & 15), (n & 15));
}


bool
opcode_valid(int c)
{
    if (c < 0 || c >= 128)
        return false;
    switch ((opcode_t)c)
    {
    case opcode_zero:
        return false;

    case opcode_plus_equals:
    case opcode_minus_equals:
    case opcode_mul:
    case opcode_div:
    case opcode_sqrt:
    case opcode_ent:
    case opcode_sj:
    case opcode_ej:
    case opcode_mj:
    case opcode_uj:
    case opcode_fj:
    case opcode_suj:
    case opcode_sfj:
    case opcode_srj:
    case opcode_m1:
    case opcode_m2:
    case opcode_m3:
    case opcode_m4:
    case opcode_m5:
    case opcode_m6:
    case opcode_m7:
    case opcode_m8:
    case opcode_m9:
    case opcode_m10:
    case opcode_m11:
    case opcode_m12:
    case opcode_m13:
    case opcode_m14:
    case opcode_mm1:
    case opcode_mm2:
    case opcode_sm3:
    case opcode_sm4:
    case opcode_sm5:
    case opcode_sm6:
    case opcode_sm7:
    case opcode_sm8:
    case opcode_sm9:
    case opcode_sm10:
    case opcode_sm11:
    case opcode_sm12:
    case opcode_sm13:
    case opcode_sm14:
    case opcode_rm1:
    case opcode_rm2:
    case opcode_rm3:
    case opcode_rm4:
    case opcode_rm5:
    case opcode_rm6:
    case opcode_rm7:
    case opcode_rm8:
    case opcode_rm9:
    case opcode_rm10:
    case opcode_rm11:
    case opcode_rm12:
    case opcode_rm13:
    case opcode_rm14:
    case opcode_clear_indicator:
    case opcode_cm1:
    case opcode_cm2:
    case opcode_cm3:
    case opcode_cm4:
    case opcode_cm5:
    case opcode_cm6:
    case opcode_cm7:
    case opcode_cm8:
    case opcode_cm9:
    case opcode_cm10:
    case opcode_cm11:
    case opcode_cm12:
    case opcode_cm13:
    case opcode_cm14:
    case opcode_dot:
    case opcode_rv:
    case opcode_right:
    case opcode_chg_sign:
    case opcode_round_down:
    case opcode_round_off:
    case opcode_round_up:
    case opcode_print:
    case opcode_fd:
    case opcode_n0:
    case opcode_n1:
    case opcode_n2:
    case opcode_n3:
    case opcode_n4:
    case opcode_n5:
    case opcode_n6:
    case opcode_n7:
    case opcode_n8:
    case opcode_n9:
    case opcode_n10:
    case opcode_n11:
    case opcode_n12:
    case opcode_n13:
    case opcode_n14:
        return true;

    case opcode_extended_c:
    case opcode_extended_start:
    case opcode_extended_m3_tilde:
    case opcode_extended_sm3_tilde:
    case opcode_extended_rm3_tilde:
    case opcode_extended_cm3_tilde:
    case opcode_extended_off:
    case opcode_extended_program_print:
    case opcode_extended_display_print:
    case opcode_extended_paper_feed:
    case opcode_extended_rounding_switch_up:
    case opcode_extended_rounding_switch_off:
    case opcode_extended_rounding_switch_down:
    case opcode_extended_decimal_point_selector_0:
    case opcode_extended_decimal_point_selector_1:
    case opcode_extended_decimal_point_selector_2:
    case opcode_extended_decimal_point_selector_3:
    case opcode_extended_decimal_point_selector_4:
    case opcode_extended_decimal_point_selector_5:
    case opcode_extended_decimal_point_selector_6:
    case opcode_extended_decimal_point_selector_7:
    case opcode_extended_decimal_point_selector_8:
    case opcode_extended_decimal_point_selector_9:
    case opcode_extended_decimal_point_selector_10:
    case opcode_extended_decimal_point_selector_11:
    case opcode_extended_decimal_point_selector_12:
    case opcode_extended_decimal_point_selector_float:
    case opcode_extended_goto_000:
    case opcode_extended_goto_001:
    case opcode_extended_goto_002:
    case opcode_extended_goto_003:
    case opcode_extended_goto_004:
    case opcode_extended_goto_005:
    case opcode_extended_goto_006:
    case opcode_extended_goto_007:
    case opcode_extended_goto_008:
    case opcode_extended_goto_009:
    case opcode_extended_goto_010:
    case opcode_extended_goto_011:
    case opcode_extended_goto_012:
    case opcode_extended_goto_013:
    case opcode_extended_goto_014:
    case opcode_extended_goto_015:
    case opcode_extended_goto_100:
    case opcode_extended_goto_101:
    case opcode_extended_goto_102:
    case opcode_extended_goto_103:
    case opcode_extended_goto_104:
    case opcode_extended_goto_105:
    case opcode_extended_goto_106:
    case opcode_extended_goto_107:
    case opcode_extended_goto_108:
    case opcode_extended_goto_109:
    case opcode_extended_goto_110:
    case opcode_extended_goto_111:
    case opcode_extended_goto_112:
    case opcode_extended_goto_113:
    case opcode_extended_goto_114:
    case opcode_extended_goto_115:
    case opcode_extended_goto_200:
    case opcode_extended_goto_201:
    case opcode_extended_goto_202:
    case opcode_extended_goto_203:
    case opcode_extended_goto_204:
    case opcode_extended_goto_205:
    case opcode_extended_goto_206:
    case opcode_extended_goto_207:
    case opcode_extended_goto_208:
    case opcode_extended_goto_209:
    case opcode_extended_goto_210:
    case opcode_extended_goto_211:
    case opcode_extended_goto_212:
    case opcode_extended_goto_213:
    case opcode_extended_goto_214:
    case opcode_extended_goto_215:
    case opcode_extended_goto_300:
    case opcode_extended_goto_301:
    case opcode_extended_goto_302:
    case opcode_extended_goto_303:
    case opcode_extended_goto_304:
    case opcode_extended_goto_305:
    case opcode_extended_goto_306:
    case opcode_extended_goto_307:
    case opcode_extended_goto_308:
    case opcode_extended_goto_309:
    case opcode_extended_goto_310:
    case opcode_extended_goto_311:
    case opcode_extended_goto_312:
    case opcode_extended_goto_313:
    case opcode_extended_goto_314:
    case opcode_extended_goto_315:
    case opcode_extended_goto_400:
    case opcode_extended_goto_401:
    case opcode_extended_goto_402:
    case opcode_extended_goto_403:
    case opcode_extended_goto_404:
    case opcode_extended_goto_405:
    case opcode_extended_goto_406:
    case opcode_extended_goto_407:
    case opcode_extended_goto_408:
    case opcode_extended_goto_409:
    case opcode_extended_goto_410:
    case opcode_extended_goto_411:
    case opcode_extended_goto_412:
    case opcode_extended_goto_413:
    case opcode_extended_goto_414:
    case opcode_extended_goto_415:
    case opcode_extended_goto_500:
    case opcode_extended_goto_501:
    case opcode_extended_goto_502:
    case opcode_extended_goto_503:
    case opcode_extended_goto_504:
    case opcode_extended_goto_505:
    case opcode_extended_goto_506:
    case opcode_extended_goto_507:
    case opcode_extended_goto_508:
    case opcode_extended_goto_509:
    case opcode_extended_goto_510:
    case opcode_extended_goto_511:
    case opcode_extended_goto_512:
    case opcode_extended_goto_513:
    case opcode_extended_goto_514:
    case opcode_extended_goto_515:
    case opcode_extended_goto_600:
    case opcode_extended_goto_601:
    case opcode_extended_goto_602:
    case opcode_extended_goto_603:
    case opcode_extended_goto_604:
    case opcode_extended_goto_605:
    case opcode_extended_goto_606:
    case opcode_extended_goto_607:
    case opcode_extended_goto_608:
    case opcode_extended_goto_609:
    case opcode_extended_goto_610:
    case opcode_extended_goto_611:
    case opcode_extended_goto_612:
    case opcode_extended_goto_613:
    case opcode_extended_goto_614:
    case opcode_extended_goto_615:
    case opcode_extended_goto_700:
    case opcode_extended_goto_701:
    case opcode_extended_goto_702:
    case opcode_extended_goto_703:
    case opcode_extended_goto_704:
    case opcode_extended_goto_705:
    case opcode_extended_goto_706:
    case opcode_extended_goto_707:
    case opcode_extended_goto_708:
    case opcode_extended_goto_709:
    case opcode_extended_goto_710:
    case opcode_extended_goto_711:
    case opcode_extended_goto_712:
    case opcode_extended_goto_713:
    case opcode_extended_goto_714:
    case opcode_extended_goto_715:
    default:
        return false;
    }
}


bool
opcode_red(opcode_t c)
{
    if (c < 0 || c >= 128)
        return true;
    switch ((opcode_t)c)
    {
    case opcode_zero:
        return true;

    case opcode_plus_equals:
        return false;

    case opcode_minus_equals:
        return true;

    case opcode_mul:
    case opcode_div:
    case opcode_sqrt:
    case opcode_ent:
    case opcode_sj:
    case opcode_ej:
    case opcode_mj:
    case opcode_uj:
    case opcode_fj:
    case opcode_suj:
    case opcode_sfj:
    case opcode_srj:
    case opcode_m1:
    case opcode_m2:
    case opcode_m3:
    case opcode_m4:
    case opcode_m5:
    case opcode_m6:
    case opcode_m7:
    case opcode_m8:
    case opcode_m9:
    case opcode_m10:
    case opcode_m11:
    case opcode_m12:
    case opcode_m13:
    case opcode_m14:
        return false;

    case opcode_mm1:
    case opcode_mm2:
        return true;

    case opcode_sm3:
    case opcode_sm4:
    case opcode_sm5:
    case opcode_sm6:
    case opcode_sm7:
    case opcode_sm8:
    case opcode_sm9:
    case opcode_sm10:
    case opcode_sm11:
    case opcode_sm12:
    case opcode_sm13:
    case opcode_sm14:
    case opcode_rm1:
    case opcode_rm2:
    case opcode_rm3:
    case opcode_rm4:
    case opcode_rm5:
    case opcode_rm6:
    case opcode_rm7:
    case opcode_rm8:
    case opcode_rm9:
    case opcode_rm10:
    case opcode_rm11:
    case opcode_rm12:
    case opcode_rm13:
    case opcode_rm14:
    case opcode_clear_indicator:
    case opcode_cm1:
    case opcode_cm2:
    case opcode_cm3:
    case opcode_cm4:
    case opcode_cm5:
    case opcode_cm6:
    case opcode_cm7:
    case opcode_cm8:
    case opcode_cm9:
    case opcode_cm10:
    case opcode_cm11:
    case opcode_cm12:
    case opcode_cm13:
    case opcode_cm14:
    case opcode_dot:
    case opcode_rv:
    case opcode_right:
        return false;

    case opcode_chg_sign:
        return true;

    case opcode_round_down:
    case opcode_round_off:
    case opcode_round_up:
    case opcode_print:
    case opcode_fd:
    case opcode_n0:
    case opcode_n1:
    case opcode_n2:
    case opcode_n3:
    case opcode_n4:
    case opcode_n5:
    case opcode_n6:
    case opcode_n7:
    case opcode_n8:
    case opcode_n9:
    case opcode_n10:
    case opcode_n11:
    case opcode_n12:
    case opcode_n13:
    case opcode_n14:
        return false;

    case opcode_extended_c:
    case opcode_extended_start:
    case opcode_extended_m3_tilde:
    case opcode_extended_sm3_tilde:
    case opcode_extended_rm3_tilde:
    case opcode_extended_cm3_tilde:
    case opcode_extended_off:
    case opcode_extended_program_print:
    case opcode_extended_display_print:
    case opcode_extended_paper_feed:
    case opcode_extended_rounding_switch_up:
    case opcode_extended_rounding_switch_off:
    case opcode_extended_rounding_switch_down:
    case opcode_extended_decimal_point_selector_0:
    case opcode_extended_decimal_point_selector_1:
    case opcode_extended_decimal_point_selector_2:
    case opcode_extended_decimal_point_selector_3:
    case opcode_extended_decimal_point_selector_4:
    case opcode_extended_decimal_point_selector_5:
    case opcode_extended_decimal_point_selector_6:
    case opcode_extended_decimal_point_selector_7:
    case opcode_extended_decimal_point_selector_8:
    case opcode_extended_decimal_point_selector_9:
    case opcode_extended_decimal_point_selector_10:
    case opcode_extended_decimal_point_selector_11:
    case opcode_extended_decimal_point_selector_12:
    case opcode_extended_decimal_point_selector_float:
    case opcode_extended_goto_000:
    case opcode_extended_goto_001:
    case opcode_extended_goto_002:
    case opcode_extended_goto_003:
    case opcode_extended_goto_004:
    case opcode_extended_goto_005:
    case opcode_extended_goto_006:
    case opcode_extended_goto_007:
    case opcode_extended_goto_008:
    case opcode_extended_goto_009:
    case opcode_extended_goto_010:
    case opcode_extended_goto_011:
    case opcode_extended_goto_012:
    case opcode_extended_goto_013:
    case opcode_extended_goto_014:
    case opcode_extended_goto_015:
    case opcode_extended_goto_100:
    case opcode_extended_goto_101:
    case opcode_extended_goto_102:
    case opcode_extended_goto_103:
    case opcode_extended_goto_104:
    case opcode_extended_goto_105:
    case opcode_extended_goto_106:
    case opcode_extended_goto_107:
    case opcode_extended_goto_108:
    case opcode_extended_goto_109:
    case opcode_extended_goto_110:
    case opcode_extended_goto_111:
    case opcode_extended_goto_112:
    case opcode_extended_goto_113:
    case opcode_extended_goto_114:
    case opcode_extended_goto_115:
    case opcode_extended_goto_200:
    case opcode_extended_goto_201:
    case opcode_extended_goto_202:
    case opcode_extended_goto_203:
    case opcode_extended_goto_204:
    case opcode_extended_goto_205:
    case opcode_extended_goto_206:
    case opcode_extended_goto_207:
    case opcode_extended_goto_208:
    case opcode_extended_goto_209:
    case opcode_extended_goto_210:
    case opcode_extended_goto_211:
    case opcode_extended_goto_212:
    case opcode_extended_goto_213:
    case opcode_extended_goto_214:
    case opcode_extended_goto_215:
    case opcode_extended_goto_300:
    case opcode_extended_goto_301:
    case opcode_extended_goto_302:
    case opcode_extended_goto_303:
    case opcode_extended_goto_304:
    case opcode_extended_goto_305:
    case opcode_extended_goto_306:
    case opcode_extended_goto_307:
    case opcode_extended_goto_308:
    case opcode_extended_goto_309:
    case opcode_extended_goto_310:
    case opcode_extended_goto_311:
    case opcode_extended_goto_312:
    case opcode_extended_goto_313:
    case opcode_extended_goto_314:
    case opcode_extended_goto_315:
    case opcode_extended_goto_400:
    case opcode_extended_goto_401:
    case opcode_extended_goto_402:
    case opcode_extended_goto_403:
    case opcode_extended_goto_404:
    case opcode_extended_goto_405:
    case opcode_extended_goto_406:
    case opcode_extended_goto_407:
    case opcode_extended_goto_408:
    case opcode_extended_goto_409:
    case opcode_extended_goto_410:
    case opcode_extended_goto_411:
    case opcode_extended_goto_412:
    case opcode_extended_goto_413:
    case opcode_extended_goto_414:
    case opcode_extended_goto_415:
    case opcode_extended_goto_500:
    case opcode_extended_goto_501:
    case opcode_extended_goto_502:
    case opcode_extended_goto_503:
    case opcode_extended_goto_504:
    case opcode_extended_goto_505:
    case opcode_extended_goto_506:
    case opcode_extended_goto_507:
    case opcode_extended_goto_508:
    case opcode_extended_goto_509:
    case opcode_extended_goto_510:
    case opcode_extended_goto_511:
    case opcode_extended_goto_512:
    case opcode_extended_goto_513:
    case opcode_extended_goto_514:
    case opcode_extended_goto_515:
    case opcode_extended_goto_600:
    case opcode_extended_goto_601:
    case opcode_extended_goto_602:
    case opcode_extended_goto_603:
    case opcode_extended_goto_604:
    case opcode_extended_goto_605:
    case opcode_extended_goto_606:
    case opcode_extended_goto_607:
    case opcode_extended_goto_608:
    case opcode_extended_goto_609:
    case opcode_extended_goto_610:
    case opcode_extended_goto_611:
    case opcode_extended_goto_612:
    case opcode_extended_goto_613:
    case opcode_extended_goto_614:
    case opcode_extended_goto_615:
    case opcode_extended_goto_700:
    case opcode_extended_goto_701:
    case opcode_extended_goto_702:
    case opcode_extended_goto_703:
    case opcode_extended_goto_704:
    case opcode_extended_goto_705:
    case opcode_extended_goto_706:
    case opcode_extended_goto_707:
    case opcode_extended_goto_708:
    case opcode_extended_goto_709:
    case opcode_extended_goto_710:
    case opcode_extended_goto_711:
    case opcode_extended_goto_712:
    case opcode_extended_goto_713:
    case opcode_extended_goto_714:
    case opcode_extended_goto_715:
        return false;

    default:
        return true;
    }
}


bool
opcode_valid_label(int c)
{
    return (c > 0 && c < 127 && (c & 0x0F) != 0x0F && (c & 0xF0) != 0x10);
}
