//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PATH_JOIN_H
#define LIB_PATH_JOIN_H

#include <lib/config.h>
#include <string>


/**
  * The path_join function is used to glue path pieces together with a
  * slash.  But will lots of exceptions for leading an trailing slashes.
  *
  * @param lhs
  *     The left hand side path portion.
  * @param rhs
  *     The right hand side path portion.
  * @return
  *     a string, containing a suitable semantically joined path.
  */
std::string path_join(const std::string &lhs, const std::string &rhs);

/**
  * The path_join function is used to glue path pieces together with a
  * slash.  But will lots of exceptions for leading an trailing slashes.
  *
  * @param s1
  *     The first path portion.
  * @param s2
  *     The second path portion.
  * @param s3
  *     The third path portion.
  * @return
  *     a string, containing a suitable semantically joined path.
  */
std::string path_join(const std::string &s1, const std::string &s2,
    const std::string &s3);

/**
  * The path_join function is used to glue path pieces together with a
  * slash.  But will lots of exceptions for leading an trailing slashes.
  *
  * @param s1
  *     The first path portion.
  * @param s2
  *     The second path portion.
  * @param s3
  *     The third path portion.
  * @param s4
  *     The fourth path portion.
  * @return
  *     a string, containing a suitable semantically joined path.
  */
std::string path_join(const std::string &s1, const std::string &s2,
    const std::string &s3, const std::string &s4);

#endif // LIB_PATH_JOIN_H
