//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/assert.h>

#include <lib/number.h>
#include <lib/precision_and_rounding.h>


precision_and_rounding::~precision_and_rounding()
{
}


precision_and_rounding::precision_and_rounding() :
    precision(-1),
    round(round_off)
{
    assert(precision >= -1);
    assert(precision <= 14);
}


precision_and_rounding::precision_and_rounding(
    int a_precision,
    round_t a_round
) :
    precision(a_precision),
    round(a_round)
{
}


precision_and_rounding::precision_and_rounding(
    const precision_and_rounding &rhs
) :
    precision(rhs.precision),
    round(rhs.round)
{
}


precision_and_rounding &
precision_and_rounding::operator=(const precision_and_rounding &rhs)
{
    if (this != &rhs)
    {
        precision = rhs.precision;
        round = rhs.round;
    }
    return *this;
}


void
precision_and_rounding::apply(number &value)
{
    if (precision >= 0)
    {
        switch (round)
        {
        case round_up:
            value = value.round_up(precision);
            break;

        default:
            assert(!"unknown rounding");
        case round_off:
            value = value.round_off(precision);
            break;

        case round_down:
            value = value.round_down(precision);
            break;
        }
    }
}
