//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PRECISION_AND_ROUNDING_H
#define LIB_PRECISION_AND_ROUNDING_H

class number; // forward

/**
  * The precision_and_rounding class is used to represent the setting
  * of the Decimal Point Selector Dial and Rounding Switch.  These are
  * always applied together.
  */
class precision_and_rounding
{
public:
    /**
      * The destructor.
      */
    virtual ~precision_and_rounding();

    /**
      * The default constructor.
      */
    precision_and_rounding();

    enum round_t
    {
        /**
          * round away from zero
          */
        round_up,

        /**
          * round towards the closest integer.
          */
        round_off,

        /**
          * round towards zero
          */
        round_down,
    };

    /**
      * A constructor.
      *
      * @param precision
      *     the number of decimal places of precision (places to the
      *     rigth of the decimal point).
      * @param round
      *     the rounding direction.
      */
    precision_and_rounding(int precision, round_t round);

    /**
      * The copy constructor.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    precision_and_rounding(const precision_and_rounding &rhs);

    /**
      * The assignment operator.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    precision_and_rounding &operator=(const precision_and_rounding &rhs);

    /**
      * The apply method is used to apply this precision and rounding to
      * a number value.
      *
      * @param value
      *     The value to be rounded.
      */
    void apply(number &value);

    int get_precision(void) const { return precision; }

    round_t get_rounding(void) const { return round; }

private:
    /**
      * The precision instance variable is used to remember the number
      * of decimal places of precision (places to the rigth of the
      * decimal point), or -1 for floating point.
      */
    int precision;

    /**
      * The round instance variable is used to remember
      * the rounding direction.
      */
    round_t round;
};

#endif // LIB_PRECISION_AND_ROUNDING_H
