//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PROGRAM_CARD_FUNCTOR_H
#define LIB_PROGRAM_CARD_FUNCTOR_H

#include <lib/ac/stddef.h>
#include <string>
#include <boost/shared_ptr.hpp>

/**
  * The program_card_functor class is used to represent something to be
  * done for the data that makes up one program card worth of a Canola
  * Card System program of a 1614P.
  */
class program_card_functor
{
public:
    /**
      * The destructor.
      */
    virtual ~program_card_functor();

    /**
      * The function call operator is overloaded to implement the
      * functionality (that's why it's called a functor).
      *
      * @param card_num
      *     The number of the card within the set (1 based).
      * @param card_count
      *     The number of cards in the set.
      * @param data
      *     The base of the array of bytes containing the program.
      * @param data_size
      *     The number of bytes in this card image (> 0, <= 40).
      */
    virtual void operator()(unsigned card_num, unsigned card_count,
        const unsigned char *data, size_t data_size) = 0;

    /**
      * The instructions per card method is used to ask how many
      * instructions the functor would like per card.  The default is
      * 40, the number on the original program cards of the 11614P.
      */
    virtual size_t instructions_per_card(void) const;

    virtual void set_title(const std::string &text);
    virtual void set_arrow(bool value);
    virtual void set_color(const std::string &text);
    virtual void set_disassemble(bool value);
    virtual void set_perforations(bool value);
    virtual void set_trim_corners(bool value);
    virtual void set_style(const std::string &text);
    virtual void set_sfj_list(const std::string &text);
    virtual void set_fj_list(const std::string &text);
    virtual void set_memory_list(const std::string &text);
    virtual void set_steps(int value);

protected:
    /**
      * The default constructor.
      * For use by derived classes only.
      */
    program_card_functor();

    /**
      * The copy constructor.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    program_card_functor(const program_card_functor &rhs);

    /**
      * The assignment operator.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    program_card_functor &operator=(const program_card_functor &rhs);
};

#endif // LIB_PROGRAM_CARD_FUNCTOR_H
