//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/string.h>
#include <libexplain/output.h>

#include <lib/program_card/functor/image/pdf.h>
#include <lib/program_card/functor/image/png.h>
#include <lib/program_card/functor/image/ps.h>
#include <lib/program_card/functor/image/svg.h>
#include <lib/sizeof.h>


struct table_t
{
    const char *name;
    program_card_functor_image::pointer (*create)(const std::string &filename);
};

static const table_t table[] =
{
    { "pdf", &program_card_functor_image_pdf::create },
    { "png", &program_card_functor_image_png::create },
    { "ps", &program_card_functor_image_ps::create },
    { "svg", &program_card_functor_image_svg::create },
};


program_card_functor_image::pointer
program_card_functor_image::factory(const std::string &filename)
{
    const char *ext = strrchr(filename.c_str(), '.');
    if (ext)
        ++ext;
    else
        ext = "";
    for (const table_t *tp = table; tp < ENDOF(table); ++tp)
    {
        if (0 == strcasecmp(ext, tp->name))
            return (*tp->create)(filename);
    }

    explain_output_error_and_die("unknown output file extension \"%s\"", ext);
    return pointer();
}
