//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/math.h>
#include <cairomm/context.h>
#include <cairomm/surface.h>

#include <lib/context_saver.h>
#include <lib/program_card/functor/image/pdf.h>


program_card_functor_image_pdf::~program_card_functor_image_pdf()
{
}


program_card_functor_image::pointer
program_card_functor_image_pdf::create(const std::string &fn)
{
    return pointer(new program_card_functor_image_pdf(fn));
}


static const double page_width = 210 / 25.4 * 72;
static const double page_height = 297 / 25.4 * 72; // points
static const double card_width = 1360. / 600. * 72.; // points
static const double card_height = 4425. / 600. * 72.;
static const double gap = 72. / 4;
static const double envelope_margin = 72 * (25./600);
static const double envelope_flap = 72 * (250./600);


program_card_functor_image_pdf::program_card_functor_image_pdf(
    const std::string &fn
) :
    filename(fn)
{
    img = Cairo::PdfSurface::create(filename, page_width, page_height);
    cr = Cairo::Context::create(img);
}


void
program_card_functor_image_pdf::operator()(unsigned card_num,
    unsigned card_count, const unsigned char *data, size_t data_size)
{
    assert(card_count > 0);
    assert(card_num >= 1);
    assert(card_num <= card_count);
    set_position(card_num, card_count);

    int cards_per_page = page_height / (card_width + gap);
    int top_margin =
        (page_height - cards_per_page * (card_width + gap) + gap) / 2;

    if (card_num == 1)
    {
        context_saver cs(cr);
        double ew = 2 * (card_width + 2 * envelope_margin) + envelope_flap;
        double eh = card_height + envelope_margin + envelope_flap;
        double y = top_margin + 2 * card_width + gap;
        cr->translate((page_width + eh) / 2, y - ew);
        cr->rotate_degrees(90);
        draw_envelope(cr);
    }

    int card_y = (card_num + 1) % cards_per_page;
    if (card_y == 0)
        cr->show_page();

    double x = (page_width - card_height) / 2;
    double y = top_margin + card_y * (card_width + gap) + card_width;

    context_saver cs(cr);
    cr->translate(x, y);
    cr->rotate_degrees(-90);

    //
    // Draw the card image.
    //
    draw(cr, data, data_size);
}
