//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PDF_H
#define LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PDF_H

#include <string>
#include <cairomm/context.h>
#include <cairomm/surface.h>

#include <lib/program_card/functor/image.h>

/**
  * The program_card_functor_image_pdf class is used to represent the
  * processing required to draw program card images into a PDF file.
  */
class program_card_functor_image_pdf:
    public program_card_functor_image
{
public:
    /**
      * The destructor.
      */
    virtual ~program_card_functor_image_pdf();

    /**
      * The create class method is used to create a new dynamically
      * allocated instance of this class.
      *
      * @param filename
      *     The path of the output file to be written.
      * @returns
      *     a managed pointer to a new instance of this class.
      */
    static pointer create(const std::string &filename);

    // See base class for documentation.
    void operator()(unsigned card_num, unsigned card_count,
        const unsigned char *data, size_t data_size);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param filename
      *     The name of the file to be written.
      */
    program_card_functor_image_pdf(const std::string &filename);

    /**
      * The filename instance variable is used to remember the name of
      * the file to be created.
      */
    std::string filename;

    Cairo::RefPtr<Cairo::Surface> img;

    Cairo::RefPtr<Cairo::Context> cr;

    /**
      * The default constructor.
      * Do not use.
      */
    program_card_functor_image_pdf();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    program_card_functor_image_pdf(const program_card_functor_image_pdf &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    program_card_functor_image_pdf &operator=(
        const program_card_functor_image_pdf &rhs);
};

#endif // LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PDF_H
