//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/program_card/functor/image/pixbuf.h>


program_card_functor_image_pixbuf::~program_card_functor_image_pixbuf()
{
}


program_card_functor_image_pixbuf::program_card_functor_image_pixbuf(
    result_t &a_result,
    double a_dpi
) :
    result(a_result),
    dots_per_inch(a_dpi)
{
}


program_card_functor_image_pixbuf::pointer
program_card_functor_image_pixbuf::create(result_t &a_result, double a_dpi)
{
    return pointer(new program_card_functor_image_pixbuf(a_result, a_dpi));
}


void
program_card_functor_image_pixbuf::operator()(unsigned card_num,
    unsigned card_count, const unsigned char *data, size_t data_size)
{
    set_position(card_num, card_count);

    //
    // Create an image surface to draw on.
    //
    int w = floor(0.5 + (1360./600.) * dots_per_inch);
    int h = floor(0.5 + (4425./600.) * dots_per_inch);
    Cairo::RefPtr<Cairo::Surface> img =
        Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, w, h);
    Cairo::RefPtr<Cairo::Context> cr = Cairo::Context::create(img);

    // The card drawing functions assume units are in points, just like
    // PostScript uses.
    double s = dots_per_inch / 72.;
    cr->scale(s, s);

    //
    // Draw the card image.
    //
    draw(cr, data, data_size);

    //
    // Make a pixbuf out of the cairo image surface.
    //
    pixbuf_ptr_t p = Gdk::Pixbuf::create(img, 0, 0, w, h);
    result.push_back(p);
}
