//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PIXBUF_H
#define LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PIXBUF_H

#include <lib/config.h>
#include <gtkmm.h>

#include <lib/program_card/functor/image.h>

/**
  * The program_card_functor_image_pixbuf class is used to represent the
  * processing required to build a set of pixbufs from a program.
  */
class program_card_functor_image_pixbuf:
    public program_card_functor_image
{
public:
    /**
      * The destructor.
      */
    virtual ~program_card_functor_image_pixbuf();

    typedef Glib::RefPtr<Gdk::Pixbuf> pixbuf_ptr_t;

    typedef std::vector<pixbuf_ptr_t> result_t;

    /**
      * The create class method is used to create a new dynamically
      * allocated instance of this class.
      *
      * @param result
      *     Where to place the images once they have been created.
      * @param dpi
      *     The resolution of the containing window, dots per inch
      * @returns
      *     a managed pointer to a new instance of this class.
      */
    static pointer create(result_t &result, double dpi);

    // See base class for documentation.
    void operator()(unsigned card_num, unsigned card_count,
        const unsigned char *data, size_t data_size);

private:
    /**
      * The constructor.
      * It is private on purpose, use the #create class method instead.
      *
      * @param result
      *     Where to place the images once they have been created.
      * @param dpi
      *     The resolution of the containing window, dots per inch
      */
    program_card_functor_image_pixbuf(result_t &result, double dpi);

    result_t &result;
    double dots_per_inch;

    /**
      * The default constructor.
      * Do not use.
      */
    program_card_functor_image_pixbuf();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    program_card_functor_image_pixbuf(
        const program_card_functor_image_pixbuf &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    program_card_functor_image_pixbuf &operator=(
        const program_card_functor_image_pixbuf &rhs);
};

#endif // LIB_PROGRAM_CARD_FUNCTOR_IMAGE_PIXBUF_H
