//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/math.h>
#include <cairomm/context.h>
#include <cairomm/surface.h>

#include <lib/program_card/functor/image/png.h>
#include <lib/std_string_printf.h>


program_card_functor_image_png::~program_card_functor_image_png()
{
}


program_card_functor_image_png::program_card_functor_image_png(
    const std::string &fn
) :
    filename(fn)
{
}


program_card_functor_image::pointer
program_card_functor_image_png::create(const std::string &fn)
{
    return pointer(new program_card_functor_image_png(fn));
}


void
program_card_functor_image_png::operator()(unsigned card_num,
    unsigned card_count, const unsigned char *data, size_t data_size)
{
    set_position(card_num, card_count);
    double dots_per_inch = 300;

    //
    // Create an image surface to draw on.
    //
    int w = floor(0.5 + (1360./600.) * dots_per_inch);
    int h = floor(0.5 + (4425./600.) * dots_per_inch);
    Cairo::RefPtr<Cairo::Surface> img =
        Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, w, h);
    Cairo::RefPtr<Cairo::Context> cr = Cairo::Context::create(img);

    // The card drawing functions assume units are in points, just like
    // PostScript uses.
    double s = dots_per_inch / 72.;
    cr->scale(s, s);

    //
    // Draw the card image.
    //
    draw(cr, data, data_size);

    //
    // Write the image to the file.
    //
    std::string fn = filename;
    if (card_count > 1)
    {
        if (fn.size() > 4 && fn.substr(fn.size() - 4, 4) == ".png")
            fn = fn.substr(0, fn.size() - 4);
        fn += std_string_printf("-%u-of-%u.png", card_num, card_count);
    }
    img->write_to_png(fn);
}
