//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/math.h>
#include <cairomm/context.h>
#include <cairomm/surface.h>

#include <lib/program_card/functor/image/ps.h>


program_card_functor_image_ps::~program_card_functor_image_ps()
{
}


program_card_functor_image::pointer
program_card_functor_image_ps::create(const std::string &fn)
{
    return pointer(new program_card_functor_image_ps(fn));
}


static double page_width = 210 / 25.4 * 72;
static double page_height = 297 / 25.4 * 72; // points
static double card_width = 1360. / 600. * 72.; // points
static double card_height = 4425. / 600. * 72.;
static double gap = 72. / 4;


program_card_functor_image_ps::program_card_functor_image_ps(
    const std::string &fn
) :
    filename(fn)
{
    img = Cairo::PsSurface::create(filename, page_width, page_height);
    cr = Cairo::Context::create(img);

    // pcd.set_card_num(card_num);
    // pcd.set_card_count(card_count);
    // FIXME: add more style options
    // what colour?
    // with an arrow?
    // with instruction disassembly on right hand side?
    // white holes or black holes?
    // blank or with "perforations"?
}


void
program_card_functor_image_ps::operator()(unsigned card_num,
    unsigned card_count, const unsigned char *data, size_t data_size)
{
    set_position(card_num, card_count);

    assert(card_count > 0);
    assert(card_num >= 1);
    assert(card_num <= card_count);
    --card_num;

    int cards_per_page = page_height / (card_width + gap);
    int top_margin = (page_height - cards_per_page * (card_width + gap)) / 2;
    if (card_num && 0 == (card_num % cards_per_page))
    {
        cr->show_page();
    }
    card_num %= cards_per_page;

    double x = (page_width - card_height) / 2;
    double y = top_margin + card_num * (card_width + gap) + card_width;

    cr->save();
    cr->translate(x, y);
    cr->rotate_degrees(-90);

    //
    // Draw the card image.
    //
    draw(cr, data, data_size);
    cr->restore();
}
