//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/math.h>
#include <cairomm/context.h>
#include <cairomm/surface.h>

#include <lib/program_card/functor/image/svg.h>
#include <lib/std_string_printf.h>


program_card_functor_image_svg::~program_card_functor_image_svg()
{
}


program_card_functor_image_svg::program_card_functor_image_svg(
    const std::string &fn
) :
    filename(fn)
{
}


program_card_functor_image::pointer
program_card_functor_image_svg::create(const std::string &fn)
{
    return pointer(new program_card_functor_image_svg(fn));
}


void
program_card_functor_image_svg::operator()(unsigned card_num,
    unsigned card_count, const unsigned char *data, size_t data_size)
{
    set_position(card_num, card_count);

    // build the output file name
    std::string fn = filename;
    if (card_count > 1)
    {
        if (fn.size() > 4 && fn.substr(fn.size() - 4, 4) == ".svg")
            fn = fn.substr(0, fn.size() - 4);
        fn += std_string_printf("-%u-of-%u.svg", card_num, card_count);
    }

    //
    // Create an image surface to draw on.
    //
    double points_per_inch = 72;
    int w = floor(0.5 + (1360./600.) * points_per_inch);
    int h = floor(0.5 + (4425./600.) * points_per_inch);
    Cairo::RefPtr<Cairo::Surface> img = Cairo::SvgSurface::create(fn, w, h);
    Cairo::RefPtr<Cairo::Context> cr = Cairo::Context::create(img);

    //
    // Draw the card image.
    //
    draw(cr, data, data_size);
}
