//
// canola - canon canola 1614p emulator
// Copyright (C) 2011, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/errno.h>
#include <sys/stat.h>
#include <libexplain/stat.h>
#include <libexplain/output.h>

#include <lib/libdir.h>
#include <lib/path_join.h>
#include <lib/search_path.h>


search_path::~search_path()
{
}


search_path::search_path() :
    have_system(false)
{
}


search_path::search_path(const std::string &a_subdir, bool dot) :
    subdir(a_subdir),
    have_system(false)
{
    if (dot)
        path.push_back(".");
}


static bool
dir_exists(const std::string &path)
{
    struct stat st;
    return (stat(path.c_str(), &st) >= 0 && S_ISDIR(st.st_mode));
}


void
search_path::append(const std::string &dir)
{
    path.push_back(dir);

    if (!subdir.empty())
    {
        std::string s = path_join(dir, subdir);
        if (dir_exists(s))
            path.push_back(s);
    }
}


std::string
search_path::find(const std::string &filename)
{
    if (!have_system)
    {
        have_system = true;
        std::string dir = configured_datadir();
        dir = path_join(dir, "canola");
        append(dir);
    }

    for (path_t::const_iterator it = path.begin(); it != path.end(); ++it)
    {
        std::string fn = path_join(*it, filename);
        struct stat st;
        if (stat(fn.c_str(), &st) >= 0)
            return fn;
        int err = errno;
        if (err != ENOENT)
        {
            explain_output_error_and_die
            (
                "%s",
                explain_stat(fn.c_str(), &st)
            );
        }
    }

    if (path.front() == ".")
        return filename;
    std::string fn = path_join(path.back(), filename);
    struct stat st;
    explain_output_error_and_die
    (
        "%s",
        explain_errno_stat(ENOENT, fn.c_str(), &st)
    );
    return fn;
}
