//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef LIB_SEARCH_PATH_H
#define LIB_SEARCH_PATH_H

#include <lib/config.h>
#include <string>
#include <list>

/**
  * The search_path class is used to represent the processing required
  * to walk a list of directories, looking for the specified file.
  */
class search_path
{
public:
    /**
      * The destructor.
      * It is not virtual, thou shalt not derive from this class.
      */
    ~search_path();

    /**
      * The default constructor.
      */
    search_path();

    /**
      * A constructor.
      *
      * @param subdir
      *     A sub-directory to look for, and look into.
      * @param dot
      *     Always incldude the current directory at the start of the path.
      */
    search_path(const std::string &subdir, bool dot = false);

    /**
      * The append method is used to appeand a directory to the end of
      * the search path.
      *
      * @param dir
      *     The directory of interest.
      */
    void append(const std::string &dir);

    /**
      * The find method is used to locate the named file in the
      * directory search path.
      *
      * @param filename
      *     The name of the file to be searched for,
      * @returns
      *     The resolved path of the file.  If the file could not be
      *     found, the string returned is the last directory in the
      *     search path added to the filename.
      */
    std::string find(const std::string &filename);

private:
    /**
      * The subdir instance variable is used to remember the name of
      * the subdirectory that may be present in each of the diretcories
      * nominated to be searched.
      */
    std::string subdir;

    /**
      * The have_system instance variable is used to remember whether or
      * not the ${prefix}/share/{progrname} directory has been appeneded
      * to the end of the search path.
      */
    bool have_system;

    typedef std::list<std::string> path_t;

    /**
      * The path instance variable is used to remember the ordered list
      * of directories to be search for files.
      */
    path_t path;

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    search_path(const search_path &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    search_path &operator=(const search_path &rhs);
};

#endif // LIB_SEARCH_PATH_H
