#!/bin/sh
#
# canola - canon canola 1614p emulator
# Copyright (C) 2011, 2012 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
#

TEST_SUBJECT="ms-01 sin(x)"
. test_prelude

cat > test.in << 'fubar'
LRN

fj 1
    379
    suj $201
    print

    107
    suj $201
    print

    211
    suj $201
    print

    ent
    uj 1

include ms-01.txt

OPE
START
fubar
test $? -eq 0 || no_result

# They only claim 10 digits of accuracy.
# More correct answers are
#
#                  ***.....
#    0.3255681544571566687140089357947215717989
#    0.9563047559630354813386508166184189620094
#   -0.5150380749100542100816319363981382834161
#
# revealing 12 decimals of precision, which isn't too shabby.
#
##123456789012345
cat > test.ok << 'fubar'
0.325568154457173 ◊
0.956304755963000 ◊
0.515038074910264-◊
fubar
test $? -eq 0 || no_result

vp=
ifsold="$IFS"
IFS=":$IFS"
for dir in $PATH
do
    dir2=`dirname $dir`
    vp="$vp -P$dir2/datadir"
done
IFS="$ifsold"

test_calculator $vp test.in > test.out
test $? -eq 0 || fail

diff test.ok test.out
test $? -eq 0 || fail

#
# Only definite negatives are possible.
# The functionality exercised by this test appears to work,
# no other guarantees are made.
#
pass

# vim: set ts=8 sw=4 et :
