#!/bin/sh
#
# canola - canon canola 1614p emulator
# Copyright (C) 2011 Peter Miller
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 3, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
#

TEST_SUBJECT="cbrt by halley"
. test_prelude

cat > test.in << 'fubar'
LRN
ci 7 suj $210 print
ci 8 suj $210 print
ci 9 suj $210 print
ent

include software/cube-root-halley.txt
OPE
start
fubar
test $? -eq 0 || no_result


# cbrt(7) ~= 1.91293118277238910119911683954876028286243905034588
#            k.kkkkkkkkkkkkkkk...
#
# cbrt(9) ~= 2.08008382305190411453005682435788538633780534037326
#            k.kkkkkkkkkkkkkkk...
#
# Wow!  This is 3 places better than the Newton-Raphson method.
#
cat > test.ok << 'fubar'
1.912931182772389 ◊
               2. ◊
2.080083823051904 ◊
fubar
test $? -eq 0 || no_result

test_calculator $vpath test.in > test.out
test $? -eq 0 || fail

diff test.ok test.out
test $? -eq 0 || fail

#
# Only definite negatives are possible.
# The functionality exercised by this test appears to work,
# no other guarantees are made.
#
pass

# vim: set ts=8 sw=4 et :
