//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <lib/ac/stdlib.h>
#include <lib/ac/unistd.h>
#include <libexplain/output.h>
#include <libexplain/program_name.h>
#include <libexplain/strtol.h>

#include <lib/image.h>
#include <lib/print_version.h>


static void
usage(void)
{
    const char *prog = explain_program_name_get();
    fprintf(stderr, "Usage: %s [ <option>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


static inline int
min(int a, int b)
{
    return (a < b ? a : b);
}


int
main(int argc, char **argv)
{
    int width = 64;
    int height = 64;
    const char *outfile = 0;
    for (;;)
    {
        int c = getopt(argc, argv, "h:o:Vw:");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'h':
            height = explain_strtol_or_die(optarg, 0, 0);
            break;

        case 'V':
            print_version();
            return 0;

        case 'o':
            outfile = optarg;
            break;

        case 'w':
            width = explain_strtol_or_die(optarg, 0, 0);
            break;

        default:
            usage();
        }
    }
    if (width < 16 || height < 16)
    {
        explain_output_error_and_die
        (
            "geometry %dx%d not acceptable",
            width,
            height
        );
    }
    if (!outfile)
    {
        explain_output_error_and_die
        (
            "please specify an output file using the -o<filename> option"
        );
    }
    if (optind != argc)
        usage();

    icon_pixel top(1., 1., 1., 1.);
    icon_pixel left(1., 0., 0., 1.);
    icon_pixel bottom(0., 0., 0., 1.);
    icon_pixel right(0., 1., 0., 1.);
    icon_pixel plateau(0., 0., 1., 1.);
    int border = min(height / 4, width / 4);
    image::pointer img = image::create_blank_from_file(outfile, width, height);
    for (int y = 0; y < height; ++y)
    {
        for (int x = 0; x < width; ++x)
        {
            int x2 = width - x;
            int y2 = height - y;
            icon_pixel clr;
            if (x < border && y - x > 0)
                clr = left;
            else if (x2 < border)
                clr = right;
            else if (y < border && x2 - y > 0)
                clr = top;
            else if (y2 < border)
                clr = bottom;
            else
                clr = plateau;
            img->set_pixel(x, y, clr);
        }
    }
    img->save_to_file(outfile);

    return 0;
}
