//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CANOLA_CALCULATOR_TEST_H
#define CANOLA_CALCULATOR_TEST_H

#include <lib/calculator.h>

/**
  * The calculator_test class is used to represent a kind of calculator
  * than can be used for automatic tests.
  */
class calculator_test:
    public calculator
{
public:
    /**
      * The destructor.
      */
    virtual ~calculator_test();

    /**
      * The create class method is used to create new dynamically
      * allocated instance of this class.
      */
    static pointer create(void);

    // See base class for documentation.
    void on_error_v(const char *fmt, va_list ap);

    // See base class for documentation.
    void run(void);

protected:
    // See base class for documentation.
    void derived_print(const display &value);

    // See base class for documentation.
    void derived_feed(void);

    // See base class for documentation.
    void derived_off(void);

    // See base class for documentation.
    void derived_update_display(const display &value);

    // See base class for documentation.
    void clock_tick_start_running(void);

    // See base class for documentation.
    void printer_timer_start_running(void);

    // See base class for documentation.
    void on_opcode(opcode_t op, const location::pointer &where);

private:
    /**
      * The default constructor.
      * It is private on purpose, use the #create class method instead.
      */
    calculator_test();

    /**
      * The copy constructor.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the initialization.
      */
    calculator_test(const calculator_test &rhs);

    /**
      * The assignment operator.
      * Do not use.
      *
      * @param rhs
      *     The right hand side of the assignment.
      */
    calculator_test &operator=(const calculator_test &rhs);
};

#endif // CANOLA_CALCULATOR_TEST_H
