//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

%{

#include <lib/ac/stdlib.h>
#include <lib/ac/stdio.h>
#include <libexplain/ferror.h>

#include <lib/number/z.h>

#include <test_number_z/grammar.h>
#include <test_number_z/lex.h>

#define YYDEBUG 1

%}

%token DIV
%token EOLN
%token EQ
%token GE
%token GT
%token JUNK
%token LE
%token LP
%token LT
%token MINUS
%token MOD
%token MUL
%token NE
%token NUMBER
%token PLUS
%token RP

%left LT LE GT GE EQ NE
%left PLUS MINUS
%left MUL DIV
%right unary

%union {
    number_z *lv_number;
}

%type <lv_number> expression
%type <lv_number> NUMBER

%{

int yyparse(void);


void
grammar(void)
{
    yyparse();
    lex_end();
}


extern int yydebug;


void
grammar_debugging(void)
{
    yydebug = 1;
}

%}

%%

run
    : /* empty */
    | run line
    ;

line
    : EOLN
    | expression EOLN
        {
            printf("%s\n", $1->representation().c_str());
            delete $1;
            explain_ferror_or_die(stdout);
        }
    | error EOLN
    ;

expression
    : NUMBER
        { $$ = $1; }
    | LP expression RP
        { $$ = $2; }
    | PLUS expression
        %prec unary
        { $$ = $2; }
    | MINUS expression
        %prec unary
        {
            // basically, undefined
            number_z result = number_z() - *$2;
            delete $2;
            $$ = new number_z(result);
        }
    | expression MUL expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            number_z result = lhs * rhs;
            $$ = new number_z(result);
        }
    | expression DIV expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            number_z result = lhs / rhs;
            $$ = new number_z(result);
        }
    | expression MOD expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            number_z result = lhs % rhs;
            $$ = new number_z(result);
        }
    | expression PLUS expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            number_z result = lhs + rhs;
            $$ = new number_z(result);
        }
    | expression MINUS expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            number_z result = lhs - rhs;
            $$ = new number_z(result);
        }
    | expression LT expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs < rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    | expression LE expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs <= rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    | expression GT expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs > rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    | expression GE expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs >= rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    | expression EQ expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs == rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    | expression NE expression
        {
            number_z lhs(*$1);
            delete $1;
            number_z rhs(*$3);
            delete $3;
            bool result = (lhs != rhs);
            $$ = new number_z(result ? 1u : 0u);
        }
    ;


// vim: set ts=8 sw=4 et :
