//
// canola - canon canola 1614p emulator
// Copyright (C) 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <lib/ac/stdlib.h>
#include <lib/ac/unistd.h>
#include <libexplain/fflush.h>
#include <libexplain/freopen.h>
#include <libexplain/program_name.h>

#include <lib/print_version.h>

#include <test_number_z/grammar.h>


static void
usage(void)
{
    const char *prog = explain_program_name_get();
    fprintf(stderr, "Usage: %s [ <option>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(EXIT_FAILURE);
}


int
main(int argc, char **argv)
{
    for (;;)
    {
        int c = getopt(argc, argv, "o:Vy");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'V':
            print_version();
            return 0;

        case 'y':
            grammar_debugging();
            break;

        default:
            usage();
        }
    }
    switch (argc - optind)
    {
    case 2:
        explain_freopen_or_die(argv[optind + 1], "w", stdout);
        // Fall through...

    case 1:
        explain_freopen_or_die(argv[optind], "r", stdin);
        // Fall through...

    case 0:
        break;

    default:
        usage();
    }

    grammar();
    explain_fflush_or_die(stdout);

    return EXIT_SUCCESS;
}


// vim: set ts=8 sw=4 at :
