//
// canola - canon canola 1614p emulator
// Copyright (C) 2011 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License, version 3, as
// published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/ac/stdio.h>
#include <lib/ac/stdlib.h>
#include <lib/ac/unistd.h>
#include <libexplain/program_name.h>

#include <lib/opcode.h>
#include <lib/print_version.h>


static void
usage(void)
{
    const char *prog = explain_program_name_get();
    fprintf(stderr, "Usage: %s [ <option>... ]\n", prog);
    fprintf(stderr, "       %s -V\n", prog);
    exit(1);
}


int
main(int argc, char **argv)
{
    for (;;)
    {
        int c = getopt(argc, argv, "V");
        if (c == EOF)
            break;
        switch (c)
        {
        case 'V':
            print_version();
            return 0;

        default:
            usage();
        }
    }
    if (optind != argc)
        usage();

    printf("<html>\n");
    printf("<body>\n");
    printf("<h1> Canon Canola 1614P Opcode Table </h1>\n");
    printf("<table>\n");
    printf("<tr><th></th>");
    for (int col = 0; col < 8; ++col)
        printf("<th>%d</th>", col);
    printf("</tr>\n");
    for (int row = 0; row < 16; ++row)
    {
        printf("<tr>\n");
        printf("<th>%d:</th>\n", row);
        for (int col = 0; col < 8; ++col)
        {
            int n = (col << 4) + row;
            printf("<td align=\"center\">\n");
            if (opcode_valid(n))
            {
                printf("%s\n", opcode_name(opcode_t(n)).c_str());
            }
            printf("</td>\n");
        }
        printf("</tr>\n");
    }
    printf("</table>\n");
    printf("</body>\n");
    printf("</html>\n");

    return 0;
}
